SystemOrganization addCategory: #'SqueakNxt-Compiler-ProgramsViejo'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-TranslationViejo'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-Api'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-Hierarchy'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-Programs'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-Exceptions'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-Translators'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-Output'!

!String methodsFor: '*SqueakNxt-Compiler' stamp: 'Richo 2/21/2011 18:02'!
asNxcIdentifier
^ self inject: '' into: [:subTotal :next |
	next isAlphaNumeric 
		ifTrue: [subTotal, next asString]
		ifFalse: [subTotal , '_', next asciiValue asString]]! !

RBProgramNodeVisitor subclass: #NxcMethodTranslator
	instanceVariableNames: 'source class'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translators'!

!NxcMethodTranslator class methodsFor: 'as yet unclassified' stamp: 'Richo 2/21/2011 14:46'!
translate: selector of: class
	^ self new translate: selector of: class! !

!NxcMethodTranslator methodsFor: 'visiting' stamp: 'Richo 2/9/2011 12:10'!
acceptAssignmentNode: anAssignmentNode
	self visitNode: anAssignmentNode variable.
	source nextPutAll: ' = '.
	self visitNode: anAssignmentNode value.
	source nextPut: $;; cr.! !

!NxcMethodTranslator methodsFor: 'visiting' stamp: 'Richo 2/9/2011 10:45'!
acceptCascadeNode: aCascadeNode
	aCascadeNode messages do: [:each | self visitNode: each. source nextPut: $;; cr].! !

!NxcMethodTranslator methodsFor: 'visiting' stamp: 'Richo 2/8/2011 17:35'!
acceptLiteralNode: aLiteralNode
	| value |
	value := aLiteralNode value.
	value isFloat 
		ifTrue: [source nextPutAll: (value printShowingDecimalPlaces: 12).
			^ self].
	value isString 
		ifTrue: [source nextPut: $"; nextPutAll: value asString; nextPut: $".
			^ self].
	source nextPutAll: value asString! !

!NxcMethodTranslator methodsFor: 'visiting' stamp: 'Richo 2/22/2011 12:25'!
acceptMessageNode: aMessageNode 
	| method receiverClass |
	receiverClass := NxcTypeCollector findTypeOf: aMessageNode receiver in: class.	
	method := receiverClass ultimateCompiledMethodAt: aMessageNode selector ifAbsent: [self error: 'Method not found'].

	"Is it a nxc primitive?"
	(method pragmaAt: #nxc:)
		ifNotNilDo: [:nxcPrim | source nextPutAll: ((NxcPrimitiveRule on: nxcPrim arguments first) applyTo: aMessageNode using: self).
			^ self].

	"Should be inlined?"
	(method pragmas anySatisfy: [:each | each keyword = #inlined])
		ifTrue: [self withClass: receiverClass do: [self visitNode: (method methodClass parseTreeFor: aMessageNode selector) body].
			source skip: -2. "Remove the extra semicolon"
			^ self].

	"If we got here, it is a regular method call and we need to signal the MethodToImplement exception."
	(MethodToImplement 
		class: receiverClass
		selector: aMessageNode selector)
			signal.
	source nextPutAll: aMessageNode selector asNxcIdentifier, '('.
	{aMessageNode receiver}, aMessageNode arguments
		do: [:each | self visitNode: each]
		separatedBy: [source nextPutAll: ', '].
	source nextPutAll: ')'! !

!NxcMethodTranslator methodsFor: 'visiting' stamp: 'Richo 2/17/2011 19:38'!
acceptMethodNode: methodNode
	self storeMethodDeclaration: methodNode.
	source cr; nextPut: ${; cr.
	methodNode body temporaries do: [:var |  
		source nextPutAll: (NxcTypeCollector findTypeOf: var in: class) asNxcType; 
			space; 
			nextPutAll: var name; 
			nextPut: $;; cr].
	self visitNode: methodNode body.
	source nextPut: $}.! !

!NxcMethodTranslator methodsFor: 'visiting' stamp: 'Richo 2/8/2011 17:36'!
acceptPragmaNode: aPragmaNode
	"Do nothing"! !

!NxcMethodTranslator methodsFor: 'visiting' stamp: 'Richo 2/21/2011 15:22'!
acceptReturnNode: aReturnNode
	source nextPutAll: 'return '.
	self visitNode: aReturnNode value.
	source nextPut: $;; cr.! !

!NxcMethodTranslator methodsFor: 'visiting' stamp: 'Richo 2/9/2011 10:45'!
acceptSequenceNode: aSequenceNode
aSequenceNode statements do: [:each | 
	self visitNode: each.
	each isMessage ifTrue: [source nextPut: $;; cr]]! !

!NxcMethodTranslator methodsFor: 'visiting' stamp: 'Richo 2/21/2011 15:50'!
acceptVariableNode: aVariableNode
	(class instVarNames includes: aVariableNode name)
		ifTrue: [(InstVarToImplement class: class var: aVariableNode) signal.
			source nextPutAll: 'self.'].
	source nextPutAll: aVariableNode name.! !

!NxcMethodTranslator methodsFor: 'testing' stamp: 'Richo 2/17/2011 19:39'!
isATask: aMethodNode 
	"To define a method as a task you should use the #task pragma"
	^ aMethodNode pragmas anySatisfy: [:pragma | pragma selector = #task]! !

!NxcMethodTranslator methodsFor: 'visiting' stamp: 'Richo 2/21/2011 16:51'!
storeMethodDeclaration: methodNode 
	| methodName |
	methodName := methodNode selector.
	methodName = #main
		ifTrue: [methodName := NxcProgramTranslator defaultMainSelector]
		ifFalse: [methodName := methodName asNxcIdentifier].
	(self isATask: methodNode)
		ifTrue: [source nextPutAll: 'task']
		ifFalse: [source nextPutAll: 'safecall ', (NxcTypeCollector findTypeOf: methodNode in: class) asNxcType].
	source nextPutAll: ' ', methodName, '(', class asNxcType, ' self'.
	methodNode arguments notEmpty
		ifTrue: [source nextPutAll: ', '].
	methodNode arguments 
		do: [:arg | source nextPutAll: (NxcTypeCollector findTypeOf: arg in: class) asNxcType, ' ', arg name] 
		separatedBy: [source nextPutAll: ', '].
	source nextPut: $)! !

!NxcMethodTranslator methodsFor: 'translating' stamp: 'Richo 2/22/2011 12:23'!
translate: selector of: aClass
	| methodNode |
	source := WriteStream on: ''.
	class := aClass.
	methodNode := (RBParser parseMethod: (class ultimateSourceCodeAt: selector ifAbsent: [self error: 'Source code not found'])).
	self visitNode: methodNode.
	^ NxcTranslatedMethod class: class node: methodNode source: source contents.! !

!NxcMethodTranslator methodsFor: 'visiting' stamp: 'Richo 2/8/2011 17:37'!
visitArguments: aNodeCollection 
	"Do nothing"! !

!NxcMethodTranslator methodsFor: 'translating' stamp: 'Richo 2/21/2011 16:10'!
withClass: aClass do: aBlock
	"Replaces the class just for running the block, then it goes back to the old class.
	Very useful for translating inlined methods, which needs to use the same source but change the current class"
	| old |
	old := class.
	class := aClass.
	aBlock value.
	class := old! !

!NxcMethodTranslator methodsFor: 'translating' stamp: 'Richo 2/22/2011 12:24'!
withSource: aWriteStream do: aBlock
	"Replaces the source stream just for running the block, then it goes back to the old source.
	Very useful for NxcRule, which needs to translate just specific expressions without messing with already translated stuff"
	| oldSource |
	oldSource := source.
	source := aWriteStream.
	aBlock value.
	source := oldSource! !

RBProgramNodeVisitor subclass: #NxcTranslatorViejo
	instanceVariableNames: 'variableInfo source pragmaMethods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

!NxcTranslatorViejo commentStamp: '<historical>' prior: 0!
A NxcTranslator is responsible for translating a subclass of NxtProgram from smalltalk to Nxc. It's public protocol is very simple, just use #translate:asProgramNamed:, which returns an instance of NxcProgram.
An even easier way of translating programs is sending #asNxcProgram to a NxtProgram subclass. For example:

	FollowLineExample asNxcProgram.
	
You can also specify a different program name:

	FollowLineExample asNxcProgram: 'Other program name'.
	
A NxcTranslator is a subclass of RBProgramNodeVisitor and it uses the visitor pattern to navigate through the parse tree translating every node into a stream called source. 
The translation of MessageNodes is done via NxcPrimitiveRules. In particular, see NxcPrimitiveRule class>>#initialize which creates a Dictionary with all the defined rules.


Instance Variables
	source:		<Stream>
	variableInfo:		<VariableInfo>

source
	- The stream in which the source will be translated.

variableInfo
	- Contains the info of the variables found in the class (inst vars, temp vars and arguments).
!

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 1/3/2011 16:42'!
acceptAssignmentNode: anAssignmentNode
	self visitNode: anAssignmentNode variable.
	source space; nextPut: $=; space.
	self visitNode: anAssignmentNode value.
	source nextPut: $;; cr.! !

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 2/1/2011 17:37'!
acceptCascadeNode: aCascadeNode
	aCascadeNode messages do: [:each | 
		self visitNode: each. 
		(each selector ~= #nxc:) ifTrue: [source nextPut: $;; cr]].! !

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 2/1/2011 14:25'!
acceptLiteralNode: aLiteralNode
	| value |
	value := aLiteralNode value.
	value isFloat 
		ifTrue: [source nextPutAll: (value printShowingDecimalPlaces: 12).
			^ self].
	value isString 
		ifTrue: [source nextPut: $"; nextPutAll: value asString; nextPut: $".
			^ self].
	source nextPutAll: value asString! !

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 2/21/2011 16:51'!
acceptMessageNode: aMessageNode
	"Is it special message #nxc:?"
	aMessageNode selector = #nxc:
		ifTrue: [source nextPutAll: aMessageNode arguments first value; cr. ^self].

	"Is it primitive rule?"
	(NxcPrimitiveRuleViejo isNxcPrimitive: aMessageNode selector)
		ifTrue: [source
				nextPutAll: ((NxcPrimitiveRuleViejo ruleFor: aMessageNode selector)
						applyTo: aMessageNode using: self).
			^ self].

	"Is it a pragma method?"
	(self pragmaMethods includesKey: aMessageNode selector)
		ifTrue: [self storePragmaMethodSend: aMessageNode.
		^self].

	"If we got here, it's a regular message send"
	source
		nextPutAll: aMessageNode selector asNxcIdentifier;
		 nextPut: $(.
	aMessageNode arguments
		do: [:each | self visitNode: each]
		separatedBy: [source nextPut: $,;
				 space].
	source nextPut: $)! !

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 2/1/2011 17:39'!
acceptMethodNode: methodNode
	self storeMethodDeclaration: methodNode.
	source cr; nextPut: ${; cr.
	methodNode allTemporaryVariables do: [:var |  
		source nextPutAll: (variableInfo typeOf: var in: methodNode selector); 
				space; 
				nextPutAll: var; 
				nextPut: $;; cr].
	self visitNode: methodNode body.
	source nextPut: $}.! !

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 1/1/2011 13:28'!
acceptPragmaNode: aPragmaNode
	"Do nothing"! !

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 1/5/2011 12:40'!
acceptReturnNode: aReturnNode
	source nextPutAll: 'return'; space.
	(aReturnNode value = (RBVariableNode named: 'self'))
		ifFalse: [self visitNode: aReturnNode value].
	source nextPut: $;; cr.! !

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 2/1/2011 17:32'!
acceptSequenceNode: aSequenceNode
aSequenceNode statements do: [:each | self visitNode: each. 
	(each isMessage and: [each selector ~= #nxc:]) ifTrue: [source nextPut: $;; cr]].! !

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 1/1/2011 03:28'!
acceptVariableNode: aVariableNode
	source nextPutAll: aVariableNode name! !

!NxcTranslatorViejo methodsFor: 'type info' stamp: 'Richo 1/3/2011 15:53'!
defaultVariableType
	"If a variable type is not defined it will use this"
	^ #int! !

!NxcTranslatorViejo methodsFor: 'type info' stamp: 'Richo 2/1/2011 13:45'!
extractInstVarsFrom: aClass into: aVariableInfo
"Adds all inst vars from the given class to the variable info."
aClass instVarNames doWithIndex: [:varName :index | | type |
	type := aClass instVarTypes at: index ifAbsent: [self defaultVariableType].
	aVariableInfo instVars add: varName -> type
	].! !

!NxcTranslatorViejo methodsFor: 'type info' stamp: 'Richo 2/1/2011 13:27'!
extractVariablesFrom: aClass into: aVariableInfo
"This method populates the variableInfo. First it adds all the instVars, then it goes over every selector and adds all the arguments and temporaries. To know the type of the variables it checks pragma definitions"
self extractInstVarsFrom: aClass into: aVariableInfo.
aClass selectors do: [:selector | 
	self extractVarsFromMethod: (aClass parseTreeFor: selector) into: aVariableInfo].
! !

!NxcTranslatorViejo methodsFor: 'type info' stamp: 'Richo 2/1/2011 13:45'!
extractVarsFromMethod: methodNode into: aVariableInfo
	"Extracts all the arguments and temporary variables from the methodNode and adds them to the variableInfo. To know the vars type it looks for #argTypes: and #tempTypes: pragmas"
	| types selector |
	selector := methodNode selector.
	types := self getTypes: #argTypes: from: methodNode.
	methodNode allArgumentVariables doWithIndex: [:varName :index | | type |
		type := types at: index ifAbsent: [self defaultVariableType].
		(aVariableInfo at: selector) add: varName -> type
		].
	types := self getTypes: #tempTypes: from: methodNode.
	methodNode allTemporaryVariables doWithIndex: [:varName :index | | type |
		type := types at: index ifAbsent: [self defaultVariableType].
		(aVariableInfo at: selector) add: varName -> type
		].! !

!NxcTranslatorViejo methodsFor: 'type info' stamp: 'Richo 1/1/2011 21:50'!
getTypes: aSymbol from: aMethodNode 
"It checks the pragmas of the method node to find type definitions"
| pragma |
pragma := aMethodNode pragmas detect: [:each | each selector = aSymbol] ifNone: [^#()].
^ (pragma arguments first value subStrings: {Character space}) collect: [:each | each asSymbol]! !

!NxcTranslatorViejo methodsFor: 'testing' stamp: 'Richo 1/27/2011 18:44'!
isATask: aMethodNode 
	"To define a method as a task you should use the #task pragma. However, the #main method is *always* a task"
	^ (aMethodNode selector = #main) or: [aMethodNode pragmas anySatisfy: [:pragma | pragma selector = #task]]! !

!NxcTranslatorViejo methodsFor: 'lookup' stamp: 'Richo 2/1/2011 17:36'!
lookup: selector in: aClass andAddTo: methods 
	"Looks for the selector in the given class and adds its method if founded. 
	Keeps looking in superclasses until NxtProgram is reached."
	| addMethod |
	addMethod := [:method | 
			methods add: method.
			"Apart from adding the method, we need to extract all needed variables"
			self extractInstVarsFrom: aClass into: variableInfo;
				 extractVarsFromMethod: method into: variableInfo].

	((Pragma allNamed: #receiver:selector:arguments: in: aClass)
		detect: [:pragma | pragma arguments second = selector]
		ifNone: [])
		ifNotNilDo: [:pragma |
			(self pragmaMethods includesKey: selector)
				ifFalse: [self pragmaMethods at: selector put: pragma.
				addMethod value: (pragma methodClass parseTreeFor: pragma selector)].
			^ self].

	(aClass selectors includes: selector)
		ifTrue: [^ addMethod value: (aClass parseTreeFor: selector)].

	aClass = NxtProgram
		ifFalse: [^ self lookup: selector in: aClass superclass andAddTo: methods].

	self error: 'Method: ' , selector printString , ' - not found'! !

!NxcTranslatorViejo methodsFor: 'pragma methods' stamp: 'Richo 2/1/2011 17:44'!
pragmaMethods
	"Pragma methods are methods defined in superclasses with the #receiver:selector:arguments: pragma. This pragma defines a special translation rule that allows the method to be used for diferent statements. For instance, if we define a method like
	
	----------------------------------------------------------------------------------------
	NxtProgram>>#foo: t1 and: t2 and: t3
		<receiver: '@arg1' selector: #bar:and: arguments: #('@arg2' '@arg3')>
		...
	----------------------------------------------------------------------------------------

	we can translate 

	5 bar: 6 and: 7.   ------>   self foo: 5 and: 6 and: 7.   ------> fooandand(5, 6, 7);
	
	It is very practical for mathematical operators and stuff."
	^ pragmaMethods ifNil: [pragmaMethods := Dictionary new]! !

!NxcTranslatorViejo methodsFor: 'type info' stamp: 'Richo 1/1/2011 21:50'!
returnTypeOf: aMethodNode 
	"It looks for the #returnType: pragma to get the return type of the method"
	| pragma |
	pragma := aMethodNode pragmas detect: [:each | each selector = #returnType:] ifNone: [^#void].
	^ pragma arguments first value asSymbol.
	! !

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 2/21/2011 16:51'!
storeMethodDeclaration: methodNode 
	| methodName |
	methodName := methodNode selector.
	(self isATask: methodNode)
		ifTrue: [source nextPutAll: 'task']
		ifFalse: [source nextPutAll: 'safecall'; space; nextPutAll: (self returnTypeOf: methodNode)].
	source space; nextPutAll: methodName asNxcIdentifier; nextPut: $(.
	methodNode arguments do: [:arg | source nextPutAll: (variableInfo typeOf: arg name in: methodName); space; nextPutAll: arg name] separatedBy: [source nextPut: $,; space].
	source nextPut: $)! !

!NxcTranslatorViejo methodsFor: 'pragma methods' stamp: 'Richo 2/21/2011 16:52'!
storePragmaMethodSend: aMessageNode
	"To translate this message we use the rule defined in the #receiver:selector:arguments: pragma.
	See #pragmaMethods comment."
	| pragma args |
	pragma := self pragmaMethods at: aMessageNode selector.
	source
		nextPutAll: pragma method selector asNxcIdentifier;
		nextPut: $(.
	args := Array new: pragma method selector numArgs.
	args at: pragma arguments first asInteger put: aMessageNode receiver.
	aMessageNode arguments doWithIndex: [:node :index |
			args at: (pragma arguments last at: index) asInteger put: node
		].
	args do: [:each | self visitNode: each]
		separatedBy: [source nextPut: $,; space].
	source nextPut: $).! !

!NxcTranslatorViejo methodsFor: 'translating' stamp: 'Richo 2/10/2011 12:50'!
translate: aClass asProgramNamed: programName 
	| methods |
	source := WriteStream on: ''.

	"Store: /* programName */"
	source nextPutAll: '/*';
		 space;
		 nextPutAll: programName;
		 space;
		 nextPutAll: '*/';
		 cr; cr.

	"Collect variables"
	variableInfo := VariableInfo new.
	self extractVariablesFrom: aClass into: variableInfo.

	"Collect all methods in this class"
	methods := aClass selectors asOrderedCollection
				collect: [:each | aClass parseTreeFor: each].

	"Lookup methods in superclasses"
	methods do: [:each | 
		each sentMessages do: [:msg |
			((msg = #nxc:) 
			or: [(NxcPrimitiveRuleViejo isNxcPrimitive: msg) 
			or: [(methods anySatisfy: [:m | m selector = msg]) "Already included"
			or: [each pragmas anySatisfy: [:p | p selector = msg]]]]) "Is one of our pragmas (#task, #argTypes:, etc.)"
				ifFalse: [self lookup: msg in: aClass superclass andAddTo: methods].
		]
	].

	"Store: variable declarations"
	variableInfo instVars
		associationsDo: [:association | 
			| name type |
			name := association key.
			type := association value.
			source nextPutAll: type;
				 space;
				 nextPutAll: name;
				 nextPut: $;;
				 cr].
	source cr.

	"Store: Methods declarations"
	methods
		do: [:each |  
			self storeMethodDeclaration: each.
			source nextPut: $;;
				 cr].
	source cr.

	"Store: Methods implementation"
	methods
		do: [:each | self visitNode: each]
		separatedBy: [source cr; cr].
	source cr.

	^ NxcTranslatedProgram name: programName source: source! !

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 1/1/2011 03:25'!
visitArguments: aNodeCollection 
	"Do nothing"! !

!NxcTranslatorViejo methodsFor: 'visiting' stamp: 'Richo 2/1/2011 17:41'!
withSource: aWriteStream visitNode: aProgramNode
"Replaces the source stream for this specific node, then it goes back to the old source.
Very useful for NxcPrimitiveRule, which needs to translate just specific expressions"
| oldSource |
oldSource := source.
source := aWriteStream.
self visitNode: aProgramNode.
source := oldSource! !

RBProgramNodeVisitor subclass: #NxcTypeCollector
	instanceVariableNames: 'selfClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translators'!

!NxcTypeCollector class methodsFor: 'as yet unclassified' stamp: 'Richo 2/9/2011 13:43'!
findTypeOf: aProgramNode in: aClass 
	^ self new findTypeOf: aProgramNode in: aClass ! !

!NxcTypeCollector methodsFor: 'visiting' stamp: 'Richo 2/10/2011 11:36'!
acceptArrayNode: anArrayNode 
	self halt! !

!NxcTypeCollector methodsFor: 'visiting' stamp: 'Richo 2/10/2011 11:37'!
acceptBlockNode: aBlockNode 
	^ NXBlock! !

!NxcTypeCollector methodsFor: 'visiting' stamp: 'Richo 2/10/2011 11:37'!
acceptCascadeNode: aCascadeNode 
	^ self visitNode: aCascadeNode messages last! !

!NxcTypeCollector methodsFor: 'visiting' stamp: 'Richo 2/10/2011 11:37'!
acceptLiteralArrayNode: aRBLiteralArrayNode 
	self halt! !

!NxcTypeCollector methodsFor: 'visiting' stamp: 'Richo 2/18/2011 15:57'!
acceptLiteralNode: aLiteralNode
	aLiteralNode value isFloat
		ifTrue: [^ NXFloat].
	aLiteralNode value isNumber
		ifTrue: [^NXLong].
	(aLiteralNode value isKindOf: Boolean)	
		ifTrue: [^ NXBoolean].
	aLiteralNode value isString
		ifTrue: [^ NXString].
	self halt! !

!NxcTypeCollector methodsFor: 'visiting' stamp: 'Richo 2/21/2011 15:57'!
acceptMessageNode: aMessageNode 
	"It looks for the pragma #returnType: in the method being called. If not found, it returns the receiver's class."
	| receiverClass method |
	receiverClass := self findTypeOf: aMessageNode receiver.
	method := receiverClass ultimateCompiledMethodAt: aMessageNode selector ifAbsent: [self error: 'Method not found'].
	
	(method pragmaAt: #returnType:)
		ifNotNilDo: [:pragma | ^ (Smalltalk at: pragma arguments first)].

	^ receiverClass! !

!NxcTypeCollector methodsFor: 'visiting' stamp: 'Richo 2/17/2011 19:20'!
acceptMethodNode: aMethodNode
	"It looks for the #returnType: pragma to get the return type of the method"
	(aMethodNode pragmas detect: [:each | each selector = #returnType:] ifNone: [nil])
		ifNotNilDo: [:pragma | ^ Smalltalk at: pragma arguments first value asSymbol].
	^ NXVoid! !

!NxcTypeCollector methodsFor: 'visiting' stamp: 'Richo 2/10/2011 11:38'!
acceptPatternBlockNode: aRBPatternBlockNode 
	self halt! !

!NxcTypeCollector methodsFor: 'visiting' stamp: 'Richo 2/10/2011 11:38'!
acceptPatternWrapperBlockNode: aRBPatternWrapperBlockNode 
	self halt! !

!NxcTypeCollector methodsFor: 'visiting' stamp: 'Richo 2/10/2011 11:38'!
acceptSequenceNode: aSequenceNode 
	^ self visitNode: aSequenceNode statements last! !

!NxcTypeCollector methodsFor: 'visiting' stamp: 'Richo 2/18/2011 13:51'!
acceptVariableNode: aVariableNode 
	| methodNode findPragma className |
	methodNode := aVariableNode methodNode.
	findPragma := [:selector | "Looks in the pragmas for the given selector and returns the arguments value"
		(methodNode pragmas detect: [:each | each selector = selector] ifNone: [])
				ifNotNilDo: [:pragma | pragma arguments first value]].

	"Is it an argument?"
	(aVariableNode methodNode arguments includes: aVariableNode)
		ifTrue: [(findPragma value: #argTypes:) ifNotNilDo: [:argTypes | 
					className := argTypes at: (methodNode arguments indexOf: aVariableNode)]].

	"Is it a temp var?"
	(aVariableNode temporaryVariables includes: aVariableNode name)
		ifTrue: [(findPragma value: #tempTypes:) ifNotNilDo: [:tempTypes |
					className := tempTypes at: (aVariableNode temporaryVariables indexOf: aVariableNode name)]].

	"Is it an inst var?"
	(selfClass instVarNames includes: aVariableNode name)
		ifTrue: [className := selfClass instVarTypes at: (selfClass instVarNames indexOf: aVariableNode name)].
	
	"In theory, this should never throw an error"
	^ Smalltalk
		at: className
		ifAbsent: [self error: 'Variable''s type not found']! !

!NxcTypeCollector methodsFor: 'finding types' stamp: 'Richo 2/21/2011 16:03'!
findTypeOf: aProgramNode
	aProgramNode = (RBVariableNode named: 'self')
		ifTrue: [^ selfClass].
	^ self visitNode: aProgramNode.
! !

!NxcTypeCollector methodsFor: 'finding types' stamp: 'Richo 2/10/2011 11:43'!
findTypeOf: aProgramNode in: aClass 
	selfClass := aClass.
	^ self findTypeOf: aProgramNode! !

!ClassDescription methodsFor: '*SqueakNxt-Compiler' stamp: 'Richo 2/21/2011 15:15'!
ultimateCompiledMethodAt: selector ifAbsent: aBlock
	"Return the compiled method at selector, deferring to superclass if necessary"
	^ self compiledMethodAt: selector ifAbsent:
		[superclass
			ifNil:
				[aBlock value]
			 ifNotNil:
				[superclass ultimateCompiledMethodAt: selector ifAbsent: aBlock]]! !

!RBConfigurableFormatter methodsFor: '*squeaknxt-compiler' stamp: 'Richo 1/5/2011 10:35'!
acceptPragmaNode: aPragmaNode
	"Fixed to parse pragmas correctly"
	codeStream nextPut: $<.
	self 
		formatSelectorAndArguments: aPragmaNode 
		firstSeparator: [ 
			aPragmaNode isBinary
				ifTrue: [ self space ] ]
		restSeparator: [ self space ].
	codeStream nextPut: $>! !

!RBScanner methodsFor: '*squeaknxt-compiler' stamp: 'Richo 1/11/2011 09:37'!
scanNumber
	"Changed to use Number>>#readFrom: instead of SqNumberParser>>#parse:. For Etoys compatibility"
	| start number stop string |
	start := stream position.
	stream position: start - 1.
	(Smalltalk includesKey: #SqNumberParser)
		ifTrue: [number := (Smalltalk at: #SqNumberParser) parse: stream]
		ifFalse: [number := Number readFrom: stream].
	stop := stream position.
	stream position: start - 1.
	string := stream next: stop - start + 1.
	stream position: stop.
	self step.
	^RBNumberLiteralToken 
		value: number
		start: start
		stop: stop
		source: string! !

!FileStream class methodsFor: '*squeaknxt-compiler' stamp: 'Richo 1/1/2011 16:42'!
forceNewFileNamed: fileName do: aBlock
	"Avi Bryant says, ''This idiom is quite common in other languages that make heavy use of closures (i.e. Lisp (with-file 'foo' (f) ...) and Ruby (File.open('foo'){|f|...})).  It's time Squeak had it, too.''
	
	Returns the result of aBlock."
	
	^ self detectFile: [ self forceNewFileNamed: fileName ] do: aBlock! !

Object subclass: #NXObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Hierarchy'!

NXObject subclass: #NXBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Hierarchy'!

!NXBlock methodsFor: 'nxc-controlling' stamp: 'Richo 2/17/2011 13:21'!
doWhileTrue: aBlock
<nxc: 'do\{{0}\}while({1})'>! !

!NXBlock methodsFor: 'nxc-controlling' stamp: 'Richo 2/10/2011 11:31'!
repeat
<nxc: 'while(true)\{{0}\}'>! !

!NXBlock methodsFor: 'nxc-controlling' stamp: 'Richo 2/10/2011 11:30'!
whileTrue
<nxc: 'while({0})'>! !

!NXBlock methodsFor: 'nxc-controlling' stamp: 'Richo 2/10/2011 11:31'!
whileTrue: aBlock
<nxc: 'while({0})\{{1}\}'>! !

!NXObject class methodsFor: 'as yet unclassified' stamp: 'Richo 2/16/2011 18:37'!
asNxcType
	^ self name! !

!NXObject class methodsFor: 'definition' stamp: 'Richo 2/18/2011 13:43'!
definition
	"Answer a String that defines the receiver."
	"Overwritten to show the #instanceVariablesTypes: message in the class definition of NxtProgram subclasses"
	| aStream path |
	self = NxtProgram ifTrue: [^super definition].
	aStream := WriteStream
				on: (String new: 300).
	superclass == nil
		ifTrue: [aStream nextPutAll: 'ProtoObject']
		ifFalse: [path := ''.
			self environment
				scopeFor: superclass name
				from: nil
				envtAndPathIfFound: [:envt :remotePath | path := remotePath].
			aStream nextPutAll: path , superclass name].
	aStream nextPutAll: self kindOfSubclass;
		 store: self name.
	[(self hasTraitComposition
			and: [self traitComposition notEmpty])
		ifTrue: [aStream cr; tab; nextPutAll: 'uses: '; nextPutAll: self traitCompositionString]] on: MessageNotUnderstood do: ["Ignore DNU. Traits don't exist in Etoys"].
	aStream cr; tab; nextPutAll: 'instanceVariableNames: '; store: self instanceVariablesString.
	aStream cr; tab; nextPutAll: 'instanceVariableTypes: '; store: self instanceVariableTypesString.
	aStream cr; tab; nextPutAll: 'classVariableNames: '; store: self classVariablesString.
	aStream cr; tab; nextPutAll: 'poolDictionaries: '; store: self sharedPoolsString.
	aStream cr; tab; nextPutAll: 'category: '; store: (SystemOrganization categoryOfElement: self name) asString.
	superclass
		ifNil: [aStream nextPutAll: '.';
				 cr.
			aStream nextPutAll: self name.
			aStream space; nextPutAll: 'superclass: nil'].
	^ aStream contents! !

!NXObject class methodsFor: 'definition' stamp: 'Richo 2/18/2011 13:43'!
definitionST80
	"Only useful for the Etoys image"
	self = NXObject ifTrue: [^super definitionST80].
	^ self definition! !

!NXObject class methodsFor: 'definition' stamp: 'Richo 2/18/2011 13:43'!
instVarTypes 
	"Returns a collection of symbols representing the nxc type of each inst var in the order ther are defined"
	| instVarTypes |
	instVarTypes := (self instanceVariableTypesString subStrings: {Character space}) collect: [:each | each asSymbol].
	instVarTypes isEmpty ifTrue: [instVarTypes := #()].
	^ instVarTypes ! !

!NXObject class methodsFor: 'definition' stamp: 'Richo 2/18/2011 13:43'!
instanceVariableTypes: aString 
"Automatically compiles a method called #instanceVariableTypesString that returns the string given as a parameter"
self class compile: 'instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you''re doing"
^ ', aString printString, ' withBlanksCondensed' classified: #definition
! !

!NXObject class methodsFor: 'definition' stamp: 'Richo 2/18/2011 13:43'!
instanceVariableTypesString
	"This method will be automaticaly generated for subclasses whenever a subclass of NxtProgram is compiled. It returns the information of the inst vars' type. I choose to do it this way (instead of using an class inst var) to ensure the type info will be saved when we file out this class or store it using Monticello"
	^ ''! !

!NXObject class methodsFor: 'as yet unclassified' stamp: 'Richo 2/16/2011 18:33'!
subclass: aSymbol category: aString
	^ self subclass: aSymbol instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: aString! !

!NXObject class methodsFor: 'definition' stamp: 'Richo 2/18/2011 13:46'!
subclass: nameOfClass instanceVariableNames: instVarNames instanceVariableTypes: instVarTypes classVariableNames: classVarNames poolDictionaries: poolDictnames category: category1 
	"Overwritten to allow defining the inst vars' type
	info of a class"
	^ (self
		subclass: nameOfClass
		instanceVariableNames: instVarNames
		classVariableNames: classVarNames
		poolDictionaries: poolDictnames
		category: category1)
		instanceVariableTypes: instVarTypes ! !

!NXObject methodsFor: 'nxc-comparing' stamp: 'Richo 2/10/2011 11:51'!
= t1
<returnType: #NXBoolean>
<nxc: '({0} == {1})'>! !

!NXObject methodsFor: 'nxc-comparing' stamp: 'Richo 2/10/2011 11:51'!
~= t1
<returnType: #NXBoolean>
<nxc: '({0} !!= {1})'>! !

NXObject subclass: #NXPrimitiveType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Hierarchy'!

NXPrimitiveType subclass: #NXBoolean
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Hierarchy'!

!NXBoolean class methodsFor: 'as yet unclassified' stamp: 'Richo 2/16/2011 18:37'!
asNxcType
	^'bool'! !

!NXBoolean methodsFor: 'nxc-controlling' stamp: 'Richo 2/10/2011 11:54'!
and: aBlock
<nxc: '({0} && {1})'>! !

!NXBoolean methodsFor: 'nxc-controlling' stamp: 'Richo 2/11/2011 13:21'!
ifFalse: falseBlock
<nxc: 'if(!!({0}))\{{1}\}'>! !

!NXBoolean methodsFor: 'nxc-controlling' stamp: 'Richo 2/11/2011 13:21'!
ifFalse: falseBlock ifTrue: trueBlock
<nxc: 'if({0})\{{2}\}else\{{1}\}'>! !

!NXBoolean methodsFor: 'nxc-controlling' stamp: 'Richo 2/11/2011 13:21'!
ifTrue: aBlock
<nxc: 'if({0})\{{1}\}'>! !

!NXBoolean methodsFor: 'nxc-controlling' stamp: 'Richo 2/11/2011 13:21'!
ifTrue: trueBlock ifFalse: falseBlock
<nxc: 'if({0})\{{1}\}else\{{2}\}'>! !

!NXBoolean methodsFor: 'nxc-logical operations' stamp: 'Richo 2/10/2011 11:26'!
not
<nxc: '!!({0})'>! !

!NXBoolean methodsFor: 'nxc-controlling' stamp: 'Richo 2/10/2011 11:27'!
or: aBlock
<nxc: '({0} || {1})'>! !

NXPrimitiveType subclass: #NXNumber
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Hierarchy'!

NXNumber subclass: #NXFloat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Hierarchy'!

!NXFloat class methodsFor: 'as yet unclassified' stamp: 'Richo 2/18/2011 15:53'!
asNxcType
	^'float'! !

NXNumber subclass: #NXLong
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Hierarchy'!

!NXLong class methodsFor: 'as yet unclassified' stamp: 'Richo 2/18/2011 15:53'!
asNxcType
	^'long'! !

!NXNumber methodsFor: 'nxc-arithmetic' stamp: 'Richo 2/9/2011 18:00'!
* aNumber
<nxc: '({0} * {1})'>! !

!NXNumber methodsFor: 'nxc-arithmetic' stamp: 'Richo 2/9/2011 13:36'!
+ aNumber
<nxc: '({0} + {1})'>! !

!NXNumber methodsFor: 'nxc-arithmetic' stamp: 'Richo 2/9/2011 18:00'!
- aNumber
<nxc: '({0} - {1})'>! !

!NXNumber methodsFor: 'nxc-arithmetic' stamp: 'Richo 2/9/2011 18:02'!
/ aNumber
<nxc: '({0} / {1})'>! !

!NXNumber methodsFor: 'nxc-math functions' stamp: 'Richo 2/21/2011 16:30'!
// aNumber
<returnType: #NXLong> <argTypes: #(NXLong)>
^ (self / aNumber) floor! !

!NXNumber methodsFor: 'nxc-comparing' stamp: 'Richo 2/10/2011 11:51'!
< aNumber
<returnType: #NXBoolean>
<nxc: '({0} < {1})'>! !

!NXNumber methodsFor: 'nxc-comparing' stamp: 'Richo 2/10/2011 11:51'!
<= aNumber
<returnType: #NXBoolean>
<nxc: '({0} <= {1})'>! !

!NXNumber methodsFor: 'nxc-comparing' stamp: 'Richo 2/10/2011 11:51'!
> aNumber
<returnType: #NXBoolean>
<nxc: '({0} > {1})'>! !

!NXNumber methodsFor: 'nxc-comparing' stamp: 'Richo 2/10/2011 11:52'!
>= aNumber
<returnType: #NXBoolean>
<nxc: '({0} >= {1})'>! !

!NXNumber methodsFor: 'nxc-math functions' stamp: 'Richo 2/21/2011 16:30'!
\\ aNumber
<returnType: #NXLong> <argTypes: #(NXLong)>
^ self - (self / aNumber * aNumber)! !

!NXNumber methodsFor: 'nxc-math functions' stamp: 'Richo 2/10/2011 11:27'!
abs
<nxc: 'abs({0})'>! !

!NXNumber methodsFor: 'nxc-truncation and round off' stamp: 'Richo 2/10/2011 11:30'!
atRandom
<nxc: '(Random({0}) + 1)'>! !

!NXNumber methodsFor: 'nxc-math functions' stamp: 'Richo 2/21/2011 16:25'!
floor
<returnType: #NXLong> <argTypes: #(NXLong)>
^ self! !

!NXNumber methodsFor: 'nxc-math functions' stamp: 'Richo 2/21/2011 16:27'!
isDivisibleBy: aNumber
<returnType: #NXBoolean> <argTypes: #(NXLong)>
aNumber = 0 ifTrue: [^false].
self \\ aNumber = 0
	ifTrue: [^ true]
	ifFalse: [^ false]! !

!NXNumber methodsFor: 'nxc-arithmetic' stamp: 'Richo 2/10/2011 11:28'!
negated
<nxc: '({0} * -1)'>! !

!NXNumber methodsFor: 'nxc-truncation and round off' stamp: 'Richo 2/10/2011 11:29'!
random
<nxc: '(Random({0}) + 1)'>! !

!NXNumber methodsFor: 'nxc-math functions' stamp: 'Richo 2/21/2011 17:11'!
rounded
<returnType: #NXLong>
self >= 0 
	ifTrue: [^ (self + 0.5) truncated]
	ifFalse: [^ (self - 0.5) truncated]! !

!NXNumber methodsFor: 'nxc-math functions' stamp: 'Richo 2/10/2011 11:28'!
sign
<nxc: 'sign({0})'>! !

!NXNumber methodsFor: 'nxc-math functions' stamp: 'Richo 2/10/2011 11:28'!
sqrt
<nxc: 'sqrt({0})'>! !

!NXNumber methodsFor: 'nxc-enumerating' stamp: 'Richo 2/10/2011 11:32'!
timesRepeat: aBlock
<nxc: 'repeat({0})\{{1}\}'>! !

!NXNumber methodsFor: 'nxc-math functions' stamp: 'Richo 2/18/2011 15:59'!
truncated
<returnType: #NXLong> <argTypes: #(NXLong)>
^ self! !

NXNumber subclass: #NXUnsignedLong
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Hierarchy'!

!NXUnsignedLong class methodsFor: 'as yet unclassified' stamp: 'Richo 2/18/2011 15:54'!
asNxcType
	^ 'unsigned long'! !

!NXPrimitiveType class methodsFor: 'as yet unclassified' stamp: 'Richo 2/16/2011 18:37'!
asNxcType
	self subclassResponsibility ! !

!NXPrimitiveType class methodsFor: 'as yet unclassified' stamp: 'Richo 2/16/2011 18:30'!
definition
	"Answer a String that defines the receiver."
	"Only shows #subclass:category:"
	| aStream path |
	self = NxtProgram ifTrue: [^super definition].
	aStream := WriteStream
				on: (String new: 300).
	superclass == nil
		ifTrue: [aStream nextPutAll: 'ProtoObject']
		ifFalse: [path := ''.
			self environment
				scopeFor: superclass name
				from: nil
				envtAndPathIfFound: [:envt :remotePath | path := remotePath].
			aStream nextPutAll: path , superclass name].
	aStream nextPutAll: self kindOfSubclass;
		 store: self name.
	aStream cr; tab; nextPutAll: 'category: '; store: (SystemOrganization categoryOfElement: self name) asString.
	superclass
		ifNil: [aStream nextPutAll: '.';
				 cr.
			aStream nextPutAll: self name.
			aStream space; nextPutAll: 'superclass: nil'].
	^ aStream contents! !

!NXPrimitiveType class methodsFor: 'as yet unclassified' stamp: 'Richo 2/16/2011 18:30'!
definitionST80
	"Only useful for the Etoys image"
	^ self definition! !

!NXPrimitiveType class methodsFor: 'as yet unclassified' stamp: 'Richo 2/16/2011 18:33'!
subclass: aSymbol instanceVariableNames: instVars classVariableNames: classVars poolDictionaries: poolDicts category: aString
	^ super subclass: aSymbol instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: aString! !

NXPrimitiveType subclass: #NXString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Hierarchy'!

!NXString class methodsFor: 'as yet unclassified' stamp: 'Richo 2/16/2011 18:37'!
asNxcType
	^'string'! !

!NXString methodsFor: 'nxc-concatenation' stamp: 'Richo 2/10/2011 11:33'!
, aNXString
<nxc: 'StrCat({0}, {1})'>! !

NXPrimitiveType subclass: #NXVoid
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Hierarchy'!

!NXVoid class methodsFor: 'as yet unclassified' stamp: 'Richo 2/17/2011 19:19'!
asNxcType
	^ 'void'! !

NXObject subclass: #NXProgram
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgram class methodsFor: 'as yet unclassified' stamp: 'Richo 2/10/2011 13:01'!
translatedToNxc
	"Useful method to easily translate a program to nxc. By default the class name is used as program name"
	^ self translatedToNxc: self name! !

!NXProgram class methodsFor: 'as yet unclassified' stamp: 'Richo 2/22/2011 11:12'!
translatedToNxc: programName 
	"Useful method to easily translate a program to nxc"
	^ NxcProgramTranslator translate: self asProgramNamed: programName! !

!NXProgram methodsFor: 'primitives-buttons' stamp: 'Richo 2/10/2011 11:55'!
buttonPressed: arg1 
<returnType: #NXBoolean>
<nxc: 'ButtonPressed({1}, false)'>! !

!NXProgram methodsFor: 'primitives-display' stamp: 'Richo 2/9/2011 12:50'!
clearScreen 
<nxc: 'ClearScreen()'>! !

!NXProgram methodsFor: 'primitives-command' stamp: 'Richo 2/18/2011 15:54'!
currentTick 
<returnType: #NXUnsignedLong>
<nxc: 'CurrentTick()'>! !

!NXProgram methodsFor: 'primitives-display' stamp: 'Richo 2/9/2011 13:51'!
displayNumber: arg1 x: arg2 y: arg3 
<nxc: 'NumOut({2}, {3}, {1})'>! !

!NXProgram methodsFor: 'primitives-display' stamp: 'Richo 2/9/2011 12:50'!
displayNumber: arg1 x: arg2 y: arg3 options: arg4 
<nxc: 'NumOut({2}, {3}, {1}, {4})'>! !

!NXProgram methodsFor: 'primitives-display' stamp: 'Richo 2/9/2011 12:50'!
displayText: arg1 x: arg2 y: arg3 
<nxc: 'TextOut({2}, {3}, {1})'>! !

!NXProgram methodsFor: 'primitives-output' stamp: 'Richo 2/9/2011 12:50'!
float: arg1 
<nxc: 'Float({1})'>! !

!NXProgram methodsFor: 'abstract' stamp: 'Richo 2/17/2011 19:42'!
main
	self subclassResponsibility! !

!NXProgram methodsFor: 'primitives-output' stamp: 'Richo 2/18/2011 15:56'!
motorPower: arg1 
<returnType: #NXLong>
<nxc: 'MotorPower({1})'>! !

!NXProgram methodsFor: 'primitives-output' stamp: 'Richo 2/18/2011 15:56'!
motorSpeed: arg1 
<returnType: #NXLong>
<nxc: 'MotorActualSpeed({1})'>! !

!NXProgram methodsFor: 'primitives-output' stamp: 'Richo 2/9/2011 12:50'!
off: arg1 
<nxc: 'Off({1})'>! !

!NXProgram methodsFor: 'primitives-output' stamp: 'Richo 2/9/2011 12:50'!
onFwd: arg1 speed: arg2 
<nxc: 'OnFwd({1}, {2})'>! !

!NXProgram methodsFor: 'primitives-output' stamp: 'Richo 2/9/2011 12:50'!
onRev: arg1 speed: arg2 
<nxc: 'OnRev({1}, {2})'>! !

!NXProgram methodsFor: 'primitives-sound' stamp: 'Richo 2/9/2011 12:50'!
playFile: arg1 
<nxc: 'PlayFile({1})'>! !

!NXProgram methodsFor: 'primitives-sound' stamp: 'Richo 2/9/2011 12:50'!
playFile: arg1 volume: arg2 repeat: arg3 
<nxc: 'PlayFileEx({1}, {2}, {3})'>! !

!NXProgram methodsFor: 'primitives-sound' stamp: 'Richo 2/9/2011 12:50'!
playTone: arg1 duration: arg2 
<nxc: 'PlayTone({1}, {2})'>! !

!NXProgram methodsFor: 'primitives-sound' stamp: 'Richo 2/9/2011 12:50'!
playTone: arg1 duration: arg2 volume: arg3 repeat: arg4 
<nxc: 'PlayToneEx({1}, {2}, {3}, {4})'>! !

!NXProgram methodsFor: 'primitives-display' stamp: 'Richo 2/9/2011 12:50'!
resetScreen 
<nxc: 'ResetScreen()'>! !

!NXProgram methodsFor: 'primitives-output' stamp: 'Richo 2/9/2011 12:50'!
rotateMotor: arg1 speed: arg2 angle: arg3 
<nxc: 'RotateMotor({1}, {2}, {3})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/10/2011 11:55'!
sensorBoolean: arg1 
<returnType: #NXBoolean>
<nxc: 'SensorBoolean({1})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/18/2011 15:56'!
sensorNormalized: arg1 
<returnType: #NXLong>
<nxc: 'SensorNormalized({1})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/18/2011 15:56'!
sensorRaw: arg1 
<returnType: #NXLong>
<nxc: 'SensorRaw({1})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/18/2011 15:56'!
sensorScaled: arg1 
<returnType: #NXLong>
<nxc: 'SensorScaled({1})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/18/2011 15:56'!
sensorType: arg1 
<returnType: #NXLong>
<nxc: 'SensorType({1})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/18/2011 15:56'!
sensorUS: arg1 
<returnType: #NXLong>
<nxc: 'SensorUS({1})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/18/2011 15:56'!
sensorValue: arg1 
<returnType: #NXLong>
<nxc: 'SensorValue({1})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/9/2011 12:50'!
setSensorLight: arg1 
<nxc: 'SetSensorLight({1})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/9/2011 12:50'!
setSensorLight: arg1 active: arg2 
<nxc: 'SetSensorLight({1}, {2})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/9/2011 12:50'!
setSensorLowSpeed: arg1 
<nxc: 'SetSensorLowspeed({1})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/9/2011 12:50'!
setSensorLowSpeed: arg1 isPowered: arg2 
<nxc: 'SetSensorLowspeed({1}, {2})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/9/2011 12:50'!
setSensorSound: arg1 
<nxc: 'SetSensorSound({1})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/9/2011 12:50'!
setSensorSound: arg1 scaling: arg2 
<nxc: 'SetSensorSound({1}, {2})'>! !

!NXProgram methodsFor: 'primitives-input' stamp: 'Richo 2/9/2011 12:50'!
setSensorTouch: arg1 
<nxc: 'SetSensorTouch({1})'>! !

!NXProgram methodsFor: 'primitives-sound' stamp: 'Richo 2/9/2011 12:50'!
stopSound 
<nxc: 'StopSound()'>! !

!NXProgram methodsFor: 'primitives-command' stamp: 'Richo 2/9/2011 12:50'!
wait: arg1 
<nxc: 'Wait({1})'>! !

NXProgram subclass: #NXProgramTest1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgramTest1 methodsFor: 'as yet unclassified' stamp: 'Richo 2/17/2011 19:42'!
main
	self displayNumber: 42 x: 0 y: 0.
	self prueba; prueba2; prueba2; prueba2! !

!NXProgramTest1 methodsFor: 'as yet unclassified' stamp: 'Richo 2/9/2011 10:54'!
prueba
	self displayNumber: 7 x: 0 y: 0! !

!NXProgramTest1 methodsFor: 'as yet unclassified' stamp: 'Richo 2/17/2011 19:45'!
prueba2
	<inlined>
	self displayNumber: 7 x: 0 y: 0; prueba.! !

NXProgram subclass: #NXProgramTest10
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgramTest10 class methodsFor: 'definition' stamp: 'Richo 2/18/2011 16:01'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NXProgramTest10 methodsFor: 'as yet unclassified' stamp: 'Richo 2/21/2011 15:59'!
main
	self displayNumber: 90.51216 truncated x: 0 y: 0.
	101.45 floor.
	self wait: 2000! !

NXProgram subclass: #NXProgramTest11
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgramTest11 class methodsFor: 'definition' stamp: 'Richo 2/21/2011 16:31'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NXProgramTest11 methodsFor: 'as yet unclassified' stamp: 'Richo 2/21/2011 17:58'!
main
	(5 isDivisibleBy: 2)
		ifTrue: [self displayText: 'True' x: 0 y: 0].
	6 \\ 2 = 0 ifTrue: [self wait: 500].

	self clearScreen.
	self displayNumber: 1092 rounded x: 0 y: 0;
		 displayNumber: 1092.015 rounded x: 0 y: 0;
		 displayNumber: 1092 rounded x: 0 y: 0.
	self wait: 2000! !

NXProgram subclass: #NXProgramTest2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgramTest2 methodsFor: 'as yet unclassified' stamp: 'Richo 2/10/2011 11:57'!
main
	self displayText: 'Richo capo' x: 10 y: 10;
		playFile: 'Woops.rso';
		turnLeft; turnRight! !

!NXProgramTest2 methodsFor: 'as yet unclassified' stamp: 'Richo 2/9/2011 13:00'!
turnLeft
<inlined>
self
		onFwd: 0 speed: 100;
		off: 2! !

!NXProgramTest2 methodsFor: 'as yet unclassified' stamp: 'Richo 2/9/2011 13:00'!
turnRight
<inlined>
self
		onFwd: 1 speed: 100;
		off: 0! !

NXProgram subclass: #NXProgramTest3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgramTest3 methodsFor: 'as yet unclassified' stamp: 'Richo 2/21/2011 16:23'!
main
	self displayNumber: 3 + 4 x: 0 y: 0! !

NXProgram subclass: #NXProgramTest4
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgramTest4 methodsFor: 'as yet unclassified' stamp: 'Richo 2/9/2011 17:48'!
main
	true ifTrue: [self off: 0].
	true ifFalse: [self off: 1].
	false ifTrue: [self off: 1] ifFalse: [self off: 0].
	false ifFalse: [self off: 2] ifTrue: [self off: 1]! !

NXProgram subclass: #NXProgramTest5
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgramTest5 methodsFor: 'as yet unclassified' stamp: 'Richo 2/11/2011 13:17'!
main
	4 = 3 ifTrue: [self off: 0; off: 1].
	true not ifFalse: [self off: 1].
	false not not ifTrue: [self off: 1] ifFalse: [self off: 0].
	4 > (5 * 2) ifFalse: [self off: 2] ifTrue: [self off: 1]! !

NXProgram subclass: #NXProgramTest6
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgramTest6 methodsFor: 'as yet unclassified' stamp: 'Richo 2/18/2011 15:56'!
main
	<tempTypes: #(NXLong)>
	| a |
	a := 2.
	self displayNumber: a * 2 x: 0 y: 0.
	self wait: 1000.
	[] repeat.! !

NXProgram subclass: #NXProgramTest7
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgramTest7 methodsFor: 'as yet unclassified' stamp: 'Richo 2/18/2011 15:56'!
bar: aNumber
	<argTypes: #(NXLong)>
	<returnType: #NXLong>
	^ 60 atRandom! !

!NXProgramTest7 methodsFor: 'as yet unclassified' stamp: 'Richo 2/18/2011 15:56'!
foo
	<returnType: #NXLong>
	^ 5! !

!NXProgramTest7 methodsFor: 'as yet unclassified' stamp: 'Richo 2/17/2011 18:38'!
main
	self displayNumber: (self foo) x: (self bar: 7) y: (self bar: 0);
		wait: 1000.! !

NXProgram subclass: #NXProgramTest8
	instanceVariableNames: 'var1'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgramTest8 class methodsFor: 'definition' stamp: 'Richo 2/18/2011 13:47'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ 'NXString' withBlanksCondensed! !

!NXProgramTest8 methodsFor: 'as yet unclassified' stamp: 'Richo 2/18/2011 13:49'!
main
	var1 := 'Richo'.
	var1 := var1 , ' ', 'Capo'.
	self wait: 2000! !

NXProgram subclass: #NXProgramTest9
	instanceVariableNames: 'var1 var2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NXProgramTest9 class methodsFor: 'definition' stamp: 'Richo 2/18/2011 16:00'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ 'NXString NXLong' withBlanksCondensed! !

!NXProgramTest9 methodsFor: 'as yet unclassified' stamp: 'Richo 2/18/2011 15:56'!
foo
	<returnType: #NXLong>
	^ var2! !

!NXProgramTest9 methodsFor: 'as yet unclassified' stamp: 'Richo 2/18/2011 15:50'!
main
	var1 := 'Richo'.
	var2 := 5 * 2 * 100.
	self displayText: var1 x: 0 y: 0.
	self wait: self foo! !

Object subclass: #Nbc
	instanceVariableNames: 'parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Api'!

!Nbc commentStamp: 'Richo 1/2/2011 16:40' prior: 0!
A Nbc is a very simple API for accessing the Next Byte Codes Compiler using FFI on Windows. This class requires the actual nbc program installed in the default directory. You can download it from here "http://http://bricxcc.sourceforge.net/nbc/". FYI, I'm using the version 1.2.1.r3 (2010-06-29).

Examples: 
1) The simplest example would be compiling into a file called "output" and reporting errors to a file called "errors":

	Nbc new
		fileName: 'example_program.nbc';
		outFile: 'output';
		errorsFile: 'errors';
		execute.
		
2) To download a program into the Lego Nxt using the usb:

	Nbc new
		fileName: 'example_program.nbc';
		portName: 'usb';
		downloadProgram;
		execute.
		
3) To use the bluetooth on COM6 instead:
	
	Nbc new
		fileName: 'example_program.nbc';
		useBluetooth;
		portName: 'COM6';
		downloadProgram;
		execute.

Instance Variables
	parameters:		<WriteStream>

parameters
	- Used to build the parameters string to be sent to the nbc program. See "nbc -help" for more information.
!

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
beQuiet
	parameters space; nextPutAll: '-q'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
decompileProgram
	parameters space; nextPutAll: '-x'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
defineMacro: aByteString 
	parameters space; nextPutAll: '-D='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
dontCompile
	parameters space; nextPutAll: '-b'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
dontIncludeSystemFile
	parameters space; nextPutAll: '-n'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
downloadProgram
	parameters space; nextPutAll: '-d'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/28/2010 18:25'!
enhancedFirmware
	parameters space; nextPutAll: '-EF'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
errorsFile: aByteString 
	parameters space; nextPutAll: '-E='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'executing' stamp: 'Richo 12/29/2010 17:57'!
execute
	"Executes the command using FFI. Windows only for now"
	| result shell |
	shell := Smalltalk at: #Win32Shell ifAbsent: [^self error: 'FFI doesn''t seem to be installed'].
	(result := (shell new 
		shellExecute:  nil
		lpOperation:  'open'
		lpFile:  'nbc.exe'
		lpParameters:  parameters contents
		lpDirectory:  nil
		nShowCmd: 0) <= 32) ifTrue: [self error: 'system error, code:', result]! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
fileName: aByteString 
	parameters space; nextPutAll: aByteString! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
includePath: aByteString 
	parameters space; nextPutAll: '-I='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'initialize' stamp: 'Richo 12/25/2010 00:44'!
initialize
	super initialize.
	parameters := WriteStream on: ''! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/29/2010 17:59'!
nbc: fileName
	parameters space; nextPutAll: '-nbc='; nextPutAll: fileName! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 1/3/2011 14:07'!
optimizationLevel: number
	parameters space; nextPutAll: '-Z'; nextPutAll: number asString! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
outFile: aByteString 
	parameters space; nextPutAll: '-O='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
portName: aByteString 
	parameters space; nextPutAll: '-S='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:48'!
useBluetooth
	parameters space; nextPutAll: '-BT'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/28/2010 18:38'!
warningsOn
	parameters space; nextPutAll: '-w'! !

Object subclass: #NxcCompiler
	instanceVariableNames: 'programName source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Api'!

!NxcCompiler commentStamp: 'Richo 12/31/2010 22:03' prior: 0!
A NxcCompiler is responsible for verifying/compiling programs as well as downloading them to the LegoNxt.

===============================
	"Code example"
	
	| program |
	
	"A simple nxc program"
	program := '
	#define MOVE_TIME   1000
	#define TURN_TIME    500

	task main()
	{
		repeat(4)
		{    
			OnFwd(OUT_AC, 75);
			Wait(MOVE_TIME);
			OnRev(OUT_C, 75);
		Wait(TURN_TIME);
		}  
		Off(OUT_AC);
	}'.
	
	"The code below will verify the program and return a collection of errors (if any)"
	NxcCompiler new
		source: program;
		verify.
	
	"The code below will download the program to the nxt using the usb cable. the resulting program will be called 'SimpleProgram'"
	NxcCompiler new
		programName: 'SimpleProgram';
		source: program;
		downloadUsingUsb.
===============================

Instance Variables
	programName:		<String>
	source:		<String>

programName
	- A name for the program. If not specified the program name will be the NxcCompiler instance's hash (not a nice name for a program, I must say).

source
	- The nxc source code of the program.
!

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 22:10'!
compileInFileNamed: outFile
	"Compiles the program into a file"
	^ self execute: (Nbc new outFile: outFile)! !

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 18:10'!
downloadUsingBluetooth: portName 
	"Compiles the program and downloads it using the bluetooth on the specified port name (COMn). Returns a collection of NbcErrors"
	^ self execute: (Nbc new portName: portName; useBluetooth; downloadProgram)! !

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 18:09'!
downloadUsingUsb
	"Compiles the program and downloads it using the usb cable. Returns a collection of NbcErrors"
	^ self execute: (Nbc new portName: 'usb'; downloadProgram)! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 12/27/2010 18:14'!
errorsFileName
	"Temporary file name to store errors during compilation"
	^ self programName , '.err'! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 1/3/2011 14:07'!
execute: nbc 
	"Create the temporary files to store the source code and the errors.
	Execute nbc.
	Wait until nbc finishes and parse errors.
	Delete temporary files.
	Return errors"
	source isNil ifTrue: [^ self error: 'Source cannot be nil'].
	FileStream forceNewFileNamed: self programFileName do: [:file | file nextPutAll: source]; 
				forceNewFileNamed: self errorsFileName do: [:file | "Do nothing. Nbc will write this file"].
	[nbc fileName: self programFileName;
		 errorsFile: self errorsFileName;
		 optimizationLevel: 2;
		 execute.
	^ self parseErrors]
		ensure: [FileDirectory default deleteFileNamed: self programFileName;
				 deleteFileNamed: self errorsFileName]! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 1/4/2011 15:33'!
parse: errorsFile
"Returns a collection with all the errors founded by the nbc compiler. The errorsFile format is similar to this:

# Error: Unknown or invalid statement
File ""D:\RICHI\Squeak images\Nxt\simpleprogram.nbc"" ; line 7
#   result sbyt
#----------------------------------------------------------
1 error during compilation
"
| errors |
errors := OrderedCollection new.
[errorsFile peek = $#] 
	whileTrue: [ | errorDescription errorPosition errorSource |
		errorDescription := errorsFile nextLine allButFirst: 2.
		errorPosition := (errorsFile nextLine subStrings: {$;}) last withBlanksTrimmed.
		errorSource := errorsFile nextLine allButFirst: 4.
		errorsFile nextLine.
		errors add: (NxcError description: errorDescription position: errorPosition source: errorSource).
		].
^errors asArray! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 1/4/2011 23:37'!
parseErrors
	"Wait until the errors file is fully written by nbc, then read it and return the errors.
	IMPORTANT: Be aware that this code relies on the fact that FileStream>>#fileNamed: returns nil when the file is being modified by another process. If this somehow changes in the future, this code will inevitably break."
	| errorsFile |
	[200 milliSeconds asDelay wait.
	(errorsFile := FileStream fileNamed: self errorsFileName) isNil] whileTrue.
	[^ self parse: errorsFile]
		ensure: [errorsFile close].
! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 1/1/2011 03:07'!
program: aNxcProgram
	source := aNxcProgram source.
	programName isNil ifTrue: [programName := aNxcProgram name]! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 12/29/2010 17:43'!
programFileName
	"Temporary file name to store the source code"
	^ self programName , '.nxc'! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 22:10'!
programName
	"If programName is not specified, the default name will be this instance's hash"
	^ programName ifNil: [self hash asString]! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:47'!
programName: aString
	programName := aString! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 17:07'!
source
	^ source! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:48'!
source: aString
	source := aString.! !

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 22:10'!
verify
	"Compiles the program and returns a collection of NbcErrors. If the collection is empty then the program is ok"
	^ self execute: Nbc new! !

Object subclass: #NxcError
	instanceVariableNames: 'description position source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Api'!

!NxcError class methodsFor: 'instance creation' stamp: 'Richo 12/25/2010 16:02'!
description: description position: position source: source
^ (self basicNew setDescription: description position: position source: source) initialize ! !

!NxcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
description
	^ description ! !

!NxcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
position
	^ position ! !

!NxcError methodsFor: 'printing' stamp: 'Richo 12/27/2010 22:08'!
printOn: aStream
aStream nextPutAll: position asString;
		 nextPut: $:;
		 space;
		 nextPutAll: description asString 
! !

!NxcError methodsFor: 'private' stamp: 'Richo 12/25/2010 16:02'!
setDescription: aByteString position: aByteString2 source: aByteString3 
	description := aByteString.
	position := aByteString2.
	source := aByteString3! !

!NxcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
source
	^ source ! !

Object subclass: #NxcInstVar
	instanceVariableNames: 'varName varType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Output'!

!NxcInstVar class methodsFor: 'instance creation' stamp: 'Richo 2/22/2011 11:23'!
name: varName type: varType
	^ (self basicNew setName: varName type: varType) initialize! !

!NxcInstVar methodsFor: 'comparing' stamp: 'Richo 2/22/2011 12:14'!
= aNxcInstVar
(aNxcInstVar isKindOf: self class) ifFalse: [^ false].
^ self name = aNxcInstVar name and: [self type = aNxcInstVar type]! !

!NxcInstVar methodsFor: 'comparing' stamp: 'Richo 2/22/2011 12:16'!
hash
^ (self name hash hashMultiply bitXor: self type hash) hashMultiply ! !

!NxcInstVar methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:07'!
name
^ varName ! !

!NxcInstVar methodsFor: 'private' stamp: 'Richo 2/22/2011 11:25'!
setName: aString type: aNXClass
	varName := aString.
	varType := aNXClass ! !

!NxcInstVar methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:07'!
type
^ varType! !

Object subclass: #NxcPrimitiveRule
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translators'!

!NxcPrimitiveRule commentStamp: 'Richo 2/22/2011 12:30' prior: 0!
A NxcPrimitiveRule is is a very simple way of defining a rule to translate a smalltalk message to a nxc primitive. Every function (such as "NumOut" or "Wait"), every operator (such as "+" or "=="), and every control structure (such as "if", "while", or "repeat") is considered to be a nxc primitive. To define a method as a nxc primitive you should use the #nxc: pragma (look at the senders of #nxc: for examples).

To create a rule simply send '#as: NxcPrimitiveRule' to a well formed string. For example:

	'NumOut({2}, {3}, {1})' as: NxcPrimitiveRule.
	'({0} + {1})' as: NxcPrimitiveRule.
	'if({0})\{{1}\}else\{{2}\}' as: NxcPrimitiveRule.
	
Once a rule is created it can be applied to a message node using a translator by sending #applyTo:using:. This mehod will then replace all '{n}' in the string with the translated arguments ({0} represents the receiver).

Instance Variables
	value:		<String>

value
	- The string that defines how to apply the rule.
!

!NxcPrimitiveRule class methodsFor: 'instance creation' stamp: 'Richo 2/11/2011 13:23'!
newFrom: aString
	^ self on: aString! !

!NxcPrimitiveRule class methodsFor: 'instance creation' stamp: 'Richo 2/8/2011 19:04'!
on: aByteString 
	^ (self basicNew setValue: aByteString) initialize ! !

!NxcPrimitiveRule methodsFor: 'applying' stamp: 'Richo 2/21/2011 16:06'!
applyTo: aMessageNode using: aNxcMethodTranslator 
	"Visits the message node receiver and arguments and replaces the '{n}' in the string with the result ({0} represents the receiver). It reinitializes the source stream for every node so that we only get the result of visiting that node"
	^ self format: ({aMessageNode receiver}, aMessageNode arguments
		collectWithIndex: [:node :index | 
		(String streamContents: [:stream | 
				aNxcMethodTranslator withSource: stream do: [aNxcMethodTranslator visitNode: node].
				self removeSemicolonIfNecessaryOn: stream for: index - 1])])! !

!NxcPrimitiveRule methodsFor: 'applying' stamp: 'Richo 2/9/2011 11:30'!
format: aCollection
	"Zero based. It goes through value increasing by 1 the enclosed expressions and then it sends regular #format:"
	| oldStream newStream |
	oldStream := value readStream.
	newStream := WriteStream on: ''.
	[oldStream atEnd]
		whileFalse: [| next |
		next := oldStream next.
		newStream nextPut: next.
		(next = ${ and: [oldStream peekBack ~= $\])
			ifTrue: [| n | "We found an enclosed expression"
			n := oldStream upTo: $}.
			newStream nextPutAll: (n asNumber + 1) asString, '}'.
			]
	].
	^ newStream contents format: aCollection
	! !

!NxcPrimitiveRule methodsFor: 'printing' stamp: 'Richo 2/8/2011 19:05'!
printOn: aStream
super printOn: aStream.
aStream nextPut: $(.
aStream nextPutAll: value.
aStream nextPut: $)! !

!NxcPrimitiveRule methodsFor: 'applying' stamp: 'Richo 2/11/2011 13:22'!
removeSemicolonIfNecessaryOn: source for: argIndex
	"Depending on the position of the argument in the rule string, it can be necessary to remove the semicolon after the statement. For now, we only leave the semicolon if the token is surrounded by curly brackets {}.
	For instance, in the rule below
	
	'while({0})\{{1}\}'
	
	The node replacing the {0} token will have its semicolon removed, whereas the {1} won't. Of course, most of the time it won't be necessary to remove any semicolon (if {0} is a literal or a variable, for instance)."
	| index newContents oldContents rule token |
	oldContents := source contents.
	rule := value withBlanksCondensed.
	
	"Do nothing if the source does not end with a semicolon"
	(oldContents size > 2 and: [oldContents endsWith: ';', String cr]) ifFalse: [^self].

	token := '{', argIndex asString ,'}'.
	index := rule findString: token.

	"Do nothing if the token is not in the rule"
	index = 0 ifTrue: [^self].

	"Do nothing if the token is not surrounded by curly brackets"
	(String 
		with: (rule at: index - 2) 
		with: (rule at: index - 1)
		with: (rule at: index + token size)
		with: (rule at: index + token size + 1)) = '\{\}'
		ifTrue: [^self].

	"Okay, you can now safely remove the semicolon"
	newContents := oldContents allButLast: 2.
	source reset; nextPutAll: newContents.! !

!NxcPrimitiveRule methodsFor: 'private' stamp: 'Richo 2/8/2011 19:05'!
setValue: aByteString 
	value := aByteString ! !

Object subclass: #NxcPrimitiveRuleViejo
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

NxcPrimitiveRuleViejo class
	instanceVariableNames: 'rules'!

!NxcPrimitiveRuleViejo commentStamp: 'Richo 2/1/2011 17:33' prior: 0!
A NxcPrimitiveRule is is a very simple way of defining a rule to translate a smalltalk message to a nxc primitive. Every function (such as "NumOut" or "Wait"), every operator (such as "+" or "=="), and every control structure (such as "if", "while", or "repeat") is considered to be a nxc primitive. 
To create a rule simply send #asNxcRule to a proper string. For example:

	'NumOut(@arg2, @arg3, @arg1)' asNxcRule.
	'(@arg0 + @arg1)' asNxcRule.
	'if(@arg0){@arg1}else{@arg2}' asNxcRule.
	
Once a rule is created it can be applied to a message node using a translator by sending #applyTo:using:. This mehod will then visit the message node receiver and arguments and replace the '@arg#' in the string with the result (@arg0 represents the receiver).
To see all the defined rules look at NxcPrimitiveRule class>>#initialize.

Instance Variables
	value:		<String>

value
	- The string that defines how to apply the rule.
!

NxcPrimitiveRuleViejo class
	instanceVariableNames: 'rules'!

!NxcPrimitiveRuleViejo class methodsFor: 'rules' stamp: 'Richo 2/11/2011 13:24'!
allRules
	^ rules values collect: [:each | each as: NxcRule]! !

!NxcPrimitiveRuleViejo class methodsFor: 'rules' stamp: 'Richo 1/4/2011 10:15'!
allRulesSelectors
	^ rules keys! !

!NxcPrimitiveRuleViejo class methodsFor: 'class initialization' stamp: 'Richo 2/1/2011 17:31'!
initialize
	"This method defines all the rules for translating nxc primitives. Rules is a Dictionary where each key is a Smalltalk selector and each value is a string which can be converted to a NxcPrimitiveRule by sending the #asNxcRule message."
	"
	self initialize.
	"
	rules := {
	
		"Input"
		#setSensorLight: -> 'SetSensorLight(@arg1)'.
		#setSensorLight:active: -> 'SetSensorLight(@arg1, @arg2)'.
		#setSensorTouch: -> 'SetSensorTouch(@arg1)'.
		#setSensorSound: -> 'SetSensorSound(@arg1)'.
		#setSensorSound:scaling: -> 'SetSensorSound(@arg1, @arg2)'.
		#setSensorLowSpeed: -> 'SetSensorLowspeed(@arg1)'.
		#setSensorLowSpeed:isPowered: -> 'SetSensorLowspeed(@arg1, @arg2)'.
		#sensorValue: -> 'SensorValue(@arg1)'.
		#sensorBoolean: -> 'SensorBoolean(@arg1)'.
		#sensorNormalized: -> 'SensorNormalized(@arg1)'.
		#sensorRaw: -> 'SensorRaw(@arg1)'.
		#sensorScaled: -> 'SensorScaled(@arg1)'.
		#sensorUS: -> 'SensorUS(@arg1)'.
		#sensorType: -> 'SensorType(@arg1)'.
		
		"Output"
		#onFwd:speed: -> 'OnFwd(@arg1, @arg2)'.
		#onRev:speed: -> 'OnRev(@arg1, @arg2)'.
		#off: -> 'Off(@arg1)'.
		#float: -> 'Float(@arg1)'.
		#rotateMotor:speed:angle: -> 'RotateMotor(@arg1, @arg2, @arg3)'.
		#motorSpeed: -> 'MotorActualSpeed(@arg1)'.
		#motorPower: -> 'MotorPower(@arg1)'.

		"Display"
		#displayNumber:x:y: -> 'NumOut(@arg2, @arg3, @arg1)'.
		#displayNumber:x:y:options: -> 'NumOut(@arg2, @arg3, @arg1, @arg4)'.
		#displayText:x:y: -> 'TextOut(@arg2, @arg3, @arg1)'.
		#clearScreen -> 'ClearScreen()'.
		#resetScreen -> 'ResetScreen()'.
						
		"Sound"
		#playFile: -> 'PlayFile(@arg1)'.
		#playFile:volume:repeat: -> 'PlayFileEx(@arg1, @arg2, @arg3)'.
		#playTone:duration: -> 'PlayTone(@arg1, @arg2)'.
		#playTone:duration:volume:repeat: -> 'PlayToneEx(@arg1, @arg2, @arg3, @arg4)'.
		#stopSound -> 'StopSound()'.

		"Buttons"
		#buttonPressed: -> 'ButtonPressed(@arg1, false)'.
			
		"Command"
		#wait: -> 'Wait(@arg1)'.
		#currentTick -> 'CurrentTick()'.
	
		"Operators"
		#+ -> '(@arg0 + @arg1)'.
		#- -> '(@arg0 - @arg1)'.
		#* -> '(@arg0 * @arg1)'.
		#/ -> '(@arg0 / @arg1)'.
		#= -> '(@arg0 == @arg1)'.
		#> -> '(@arg0 > @arg1)'.
		#< -> '(@arg0 < @arg1)'.
		#<= -> '(@arg0 <= @arg1)'.
		#>= -> '(@arg0 >= @arg1)'.
		#~= -> '(@arg0 !!= @arg1)'.
		#not -> '!!(@arg0)'.
		#and: -> '(@arg0 && @arg1)'.
		#or: -> '(@arg0 || @arg1)'.
		
		"Math (Most of this functions only work with the enhanced firmware)"
		#abs -> 'abs(@arg0)'.
		#sqrt -> 'sqrt(@arg0)'.
		#sign -> 'sign(@arg0)'.
		#negated -> '(@arg0 * -1)'.
		#random -> '(Random(@arg0) + 1)'.
		#atRandom -> '(Random(@arg0) + 1)'.

		"#sin -> 'Sin(@arg0)'.
		#cos -> 'Cos(@arg0)'.
		#arcSin -> 'Asin(@arg0)'.
		#arcCos -> 'Acos(@arg0)'.
		#arcTan -> 'Atan(@arg0)'.
		#ceiling -> 'Ceil(@arg0)'.
		#exp -> 'Exp(@arg0)'.
		#floor -> 'Floor(@arg0)'.
		#tan -> 'Tan(@arg0)'.
		#ln -> 'Log(@arg0)'.
		#log -> 'Log10(@arg0)'.
		#raisedTo: -> 'Pow(@arg0, @arg1)'.
		#truncated -> 'Trunc(@arg0)'.
		#degreeSin -> 'SinD(@arg0)'.
		#degreeCos -> 'CosD(@arg0)'.
		#degreeTan -> 'TanD(@arg0)'.
		#degreeArcTan -> 'AtanD(@arg0)'."

						
		"Control structures"
		#ifTrue:ifFalse: -> 'if(@arg0){@arg1}else{@arg2}'.
		#ifTrue: -> 'if(@arg0){@arg1}'.
		#ifFalse: -> 'if(@arg0){}else{@arg1}'.
		#whileTrue -> 'while(@arg0)'.
		#whileTrue: -> 'while(@arg0){@arg1}'.
		#timesRepeat: -> 'repeat(@arg0){@arg1}'.
		#doWhileTrue: -> 'do{@arg0}while(@arg1)'.
		#repeat -> 'while(true){@arg0}'.
		
		"Others"
		#, -> 'StrCat(@arg0, @arg1)'
	} as: Dictionary! !

!NxcPrimitiveRuleViejo class methodsFor: 'rules' stamp: 'Richo 1/2/2011 17:32'!
isNxcPrimitive: selector
	^ rules includesKey: selector! !

!NxcPrimitiveRuleViejo class methodsFor: 'instance creation' stamp: 'Richo 1/1/2011 13:00'!
on: aByteString 
	^ (self basicNew setValue: aByteString) initialize ! !

!NxcPrimitiveRuleViejo class methodsFor: 'rules' stamp: 'Richo 2/11/2011 13:24'!
ruleFor: selector
	^ (rules at: selector) as: NxcRule ! !

!NxcPrimitiveRuleViejo class methodsFor: 'rules' stamp: 'Richo 2/9/2011 12:35'!
rules
	^ rules! !

!NxcPrimitiveRuleViejo methodsFor: 'applying' stamp: 'Richo 2/21/2011 16:05'!
applyTo: aMessageNode using: aNxcTranslator
	"Visits the message node receiver and arguments and replaces the '@arg#' in the string with the result (arg0 represents the receiver). It reinitializes the source stream for every node so that we only get the result of visiting that node"
	| rule |
	rule := value.
	{aMessageNode receiver}, aMessageNode arguments
		doWithIndex: [:node :index | | token |
			token := '@arg', (index - 1) asString.
			rule := rule copyReplaceAll: token with: (String streamContents: [:stream |
				aNxcTranslator withSource: stream do: [aNxcTranslator visitNode: node].
				self removeSemicolonIfNecessaryOn: stream for: token])].
	^ rule! !

!NxcPrimitiveRuleViejo methodsFor: 'printing' stamp: 'Richo 1/2/2011 17:35'!
printOn: aStream
super printOn: aStream.
aStream nextPut: $(.
aStream nextPutAll: value.
aStream nextPut: $)! !

!NxcPrimitiveRuleViejo methodsFor: 'applying' stamp: 'Richo 2/1/2011 17:34'!
removeSemicolonIfNecessaryOn: source for: token
	"Depending on the position of the token in the rule string, it can be necessary to remove the semicolon after the statement. For now, we only leave the semicolon if the token is surrounded by curly brackets {}.
	For instance, in the rule below
	
	'while(@arg0){@arg1}'
	
	The node replacing the @arg0 token will have its semicolon removed, whereas the @arg1 won't. Of course, most of the time it won't be necessary to remove any semicolon (if @arg0 is a literal or a variable, for instance)."
	| index newContents oldContents rule |
	oldContents := source contents.
	rule := value withBlanksCondensed.
	
	"Do nothing if the source does not end with a semicolon"
	(oldContents size > 2 and: [oldContents endsWith: ';', String cr]) ifFalse: [^self].

	index := rule findString: token.

	"Do nothing if the token is not in the rule"
	index = 0 ifTrue: [^self].

	"Do nothing if the token is not surrounded by curly brackets"
	((rule at: index - 1) = ${ and: [(rule at: index + token size) = $}]) ifTrue: [^self].

	"Okay, you can now safely remove the semicolon"
	newContents := oldContents allButLast: 2.
	source reset; nextPutAll: newContents.! !

!NxcPrimitiveRuleViejo methodsFor: 'private' stamp: 'Richo 1/1/2011 13:00'!
setValue: aByteString 
	value := aByteString ! !

Object subclass: #NxcProgramTranslator
	instanceVariableNames: 'program'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translators'!

!NxcProgramTranslator commentStamp: 'Richo 2/22/2011 12:34' prior: 0!
A NxcProgramTranslator is responsible for translating a subclass of NXProgram from smalltalk to Nxc. It's public protocol is very simple, just use #translate:asProgramNamed:, which returns an instance of NxcTranslatedProgram.
An even easier way of translating programs is sending #translatedToNxc to a NXProgram subclass. For example:

	FollowLineExample translatedToNxc.
	
You can also specify a different program name:

	FollowLineExample translatedToNxc: 'Other program name'.

The translation is done recursively, starting from the #main method and translating every method call found from there. For more information, see NxcMethodTranslator comment.


Instance Variables
	program:		<NxcTranslatedProgram>

program
	- The program being translated.
!

!NxcProgramTranslator class methodsFor: 'constants' stamp: 'Richo 2/16/2011 19:01'!
defaultMainSelector
"Returns the name of the function to replace the actual main of our program. Which, by necessity it's different from the main task"
	^ #'_main_'! !

!NxcProgramTranslator class methodsFor: 'instance creation' stamp: 'Richo 2/22/2011 11:10'!
new
	self shouldNotImplement! !

!NxcProgramTranslator class methodsFor: 'instance creation' stamp: 'Richo 2/22/2011 11:13'!
translate: aNXProgram asProgramNamed: programName
	^ (self basicNew setProgram: (NxcTranslatedProgram named: programName)) initialize
			translate: aNXProgram! !

!NxcProgramTranslator methodsFor: 'private' stamp: 'Richo 2/22/2011 11:21'!
setProgram: aNxcTranslatedProgram 
	program := aNxcTranslatedProgram ! !

!NxcProgramTranslator methodsFor: 'translating' stamp: 'Richo 2/22/2011 11:32'!
translate: aNxcProgram
	program addStructFor: aNxcProgram.
	[self translateMethod: #main of: aNxcProgram]
		on: InstVarToImplement do: [:ex |
			(program addStructFor: ex ownerClass)
				addVar: (NxcInstVar name: ex varName type: ex varType).
			ex resume].
	^ program! !

!NxcProgramTranslator methodsFor: 'translating' stamp: 'Richo 2/22/2011 11:21'!
translateMethod: selector of: class
	"Recursively translates this method and all methods called from it"
	[program addMethod: (NxcMethodTranslator translate: selector of: class)]
		on: MethodToImplement
		do: [:exception | 
			self translateMethod: exception methodSelector of: exception methodClass.
			exception resume]! !

Object subclass: #NxcStruct
	instanceVariableNames: 'ownerClass variables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Output'!

!NxcStruct class methodsFor: 'instance creation' stamp: 'Richo 2/22/2011 11:29'!
for: aClass 
	^ (self basicNew setOwnerClass: aClass) initialize! !

!NxcStruct methodsFor: 'accessing' stamp: 'Richo 2/22/2011 11:30'!
addVar: aNxcInstVar 
	variables add: aNxcInstVar ! !

!NxcStruct methodsFor: 'initialize-release' stamp: 'Richo 2/22/2011 11:30'!
initialize
	super initialize.
	variables := Set new.! !

!NxcStruct methodsFor: 'accessing' stamp: 'Richo 2/22/2011 11:30'!
ownerClass
	^ ownerClass ! !

!NxcStruct methodsFor: 'private' stamp: 'Richo 2/22/2011 11:30'!
setOwnerClass: aNXClass
	ownerClass := aNXClass ! !

!NxcStruct methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:03'!
variables
	^ variables ! !

Object subclass: #NxcTranslatedMethod
	instanceVariableNames: 'methodClass methodNode methodSource'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Output'!

!NxcTranslatedMethod class methodsFor: 'instance creation' stamp: 'Richo 2/21/2011 18:33'!
class: aClass node: aMethodNode source: aByteString 
	^ (self basicNew setClass: aClass node: aMethodNode source: aByteString) initialize! !

!NxcTranslatedMethod methodsFor: 'comparing' stamp: 'Richo 2/22/2011 12:05'!
= aNxcTranslatedMethod
(aNxcTranslatedMethod isKindOf: self class) ifFalse: [^ false].
^ self source = aNxcTranslatedMethod source! !

!NxcTranslatedMethod methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:21'!
argumentTypes
	^ {self selfType}, (methodNode arguments collect: [:each | NxcTypeCollector findTypeOf: each in: methodClass])! !

!NxcTranslatedMethod methodsFor: 'comparing' stamp: 'Richo 2/22/2011 12:05'!
hash
	^ self source hash hashMultiply ! !

!NxcTranslatedMethod methodsFor: 'accessing' stamp: 'Richo 2/21/2011 18:37'!
methodDeclaration
	^ (methodSource upTo: Character cr), ';'! !

!NxcTranslatedMethod methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:21'!
methodName
	^ methodNode selector asNxcIdentifier ! !

!NxcTranslatedMethod methodsFor: 'printing' stamp: 'Richo 2/21/2011 18:48'!
printOn: aStream
	aStream nextPutAll: methodSource! !

!NxcTranslatedMethod methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:21'!
returnType
	^ NxcTypeCollector findTypeOf: methodNode in: methodClass! !

!NxcTranslatedMethod methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:22'!
selfType
	^ methodClass! !

!NxcTranslatedMethod methodsFor: 'private' stamp: 'Richo 2/21/2011 18:34'!
setClass: aClass node: aMethodNode source: aByteString
	methodClass := aClass.
	methodNode := aMethodNode.
	methodSource := aByteString! !

!NxcTranslatedMethod methodsFor: 'accessing' stamp: 'Richo 2/21/2011 18:37'!
source
	^ methodSource ! !

Object subclass: #NxcTranslatedProgram
	instanceVariableNames: 'structs methods programName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Output'!

!NxcTranslatedProgram class methodsFor: 'instance creation' stamp: 'Richo 2/22/2011 11:14'!
named: programName
	^ (self basicNew setName: programName) initialize! !

!NxcTranslatedProgram methodsFor: 'accessing' stamp: 'Richo 2/22/2011 11:22'!
addMethod: aNxcTranslatedMethod 
	methods add: aNxcTranslatedMethod ! !

!NxcTranslatedProgram methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:18'!
addStructFor: aClass 
	"Adds a struct for the given class only if a struct is not already added. Returns the struct"
	^ structs detect: [:each | each ownerClass = aClass] ifNone: [structs add: (NxcStruct for: aClass)]! !

!NxcTranslatedProgram methodsFor: 'compiling' stamp: 'Richo 2/22/2011 12:07'!
compileInFileNamed: fileName
	^ NxcCompiler new program: self; compileInFileNamed: fileName! !

!NxcTranslatedProgram methodsFor: 'compiling' stamp: 'Richo 2/22/2011 12:07'!
downloadUsingBluetooth: portName
	^ NxcCompiler new program: self; downloadUsingBluetooth: portName ! !

!NxcTranslatedProgram methodsFor: 'compiling' stamp: 'Richo 2/22/2011 12:07'!
downloadUsingUsb
	^ NxcCompiler new program: self; downloadUsingUsb! !

!NxcTranslatedProgram methodsFor: 'initialize-release' stamp: 'Richo 2/22/2011 11:15'!
initialize
	super initialize.
	structs := Set new.
	methods := Set new! !

!NxcTranslatedProgram methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:10'!
mainTaskDeclaration
	^ (self mainTaskImplementation upTo: Character cr) , ';'! !

!NxcTranslatedProgram methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:22'!
mainTaskImplementation
	^'task main()
\{
{1} program;
{2}(program);
\}' format: {
	(methods detect: [:m | m methodName = 'main']) selfType asNxcType.
	NxcProgramTranslator defaultMainSelector}

! !

!NxcTranslatedProgram methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:07'!
name
	^ programName ! !

!NxcTranslatedProgram methodsFor: 'printing' stamp: 'Richo 2/22/2011 12:04'!
printOn: aStream
	aStream nextPutAll: self source! !

!NxcTranslatedProgram methodsFor: 'private' stamp: 'Richo 2/22/2011 11:35'!
setName: aString
	programName := aString! !

!NxcTranslatedProgram methodsFor: 'accessing' stamp: 'Richo 2/22/2011 12:20'!
source
	"Store program's name"
	| source |
	source := WriteStream on: ''.
	source nextPutAll: '/* ', programName ,' */'; cr; cr.

	"Store structs definitions"
	structs do: [:struct |
		source nextPutAll: 'struct ', struct ownerClass asNxcType, ' {'; cr.
		struct variables do: [:var |
			source nextPutAll: var type asNxcType, ' ', var name, ';'; cr].
		source nextPutAll: '};'; cr.
	].
	source cr.

	"Store methods' declarations"
	source nextPutAll: self mainTaskDeclaration; cr.
	methods do: [:method |
			source nextPutAll: method methodDeclaration; cr.
		].
	source cr.

	"Store methods implementation"
	source nextPutAll: self mainTaskImplementation; cr; cr.
	methods do: [:method |
			source nextPutAll: method source]
			separatedBy: [source cr; cr].

	^ source contents! !

!NxcTranslatedProgram methodsFor: 'compiling' stamp: 'Richo 2/22/2011 12:07'!
verify
	^ NxcCompiler new program: self; verify! !

Object subclass: #NxtProgram
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgram commentStamp: 'Richo 1/2/2011 00:11' prior: 0!
A NxtProgram is an abstract class whose subclasses can be translated to the nxc language and be downloaded to a Lego Nxt robot. It only has some methods defined for documentation purposes, but they're not actually needed.

The class side is more interesting because:
	* it has a few methods that make translating more easy.
	* its class side method #definition has been changed to show an extra option that allows to specify the type of the inst vars (it will only appear for subclasses, though).!

NxtProgram subclass: #DodgeWall
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!DodgeWall methodsFor: 'as yet unclassified' stamp: 'Richo 1/11/2011 09:29'!
main
	self setSensorLowSpeed: 3.
	[
	(self sensorUS: 3) < 30
		ifTrue: [self onFwd: 0 speed: 100.
	self onFwd: 2 speed: -100]
		ifFalse: [self onFwd: 0 speed: 100.
	self onFwd: 2 speed: 100]
	] repeat ! !

NxtProgram subclass: #FollowLineExample
	instanceVariableNames: 'black'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!FollowLineExample class methodsFor: 'definition' stamp: 'Richo 1/2/2011 22:22'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ 'int' withBlanksCondensed! !

!FollowLineExample methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 14:25'!
isOnBlack
	<returnType: 'bool'>
	^ (self sensorValue: 0) < (black + 10)! !

!FollowLineExample methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 14:25'!
isOnWhite
	<returnType: 'bool'>
	^ self isOnBlack not! !

!FollowLineExample methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 14:26'!
main
	"Configure sensor"
	self setSensorLight: 0 active: true.
	
	"Get 100 samples in a second and use average as black value"
	black := 0.
	100 timesRepeat: [black := black + (self sensorValue: 0). self wait: 10].
	black := black / 100.
	
	[true] whileTrue: [
		self turnLeft.
		[self isOnWhite] whileTrue. "Wait until black"
		[self isOnBlack] whileTrue. "Wait until white"
		self turnRight.
		[self isOnWhite] whileTrue. "Wait until black"
		[self isOnBlack] whileTrue. "Wait until white"
	]! !

!FollowLineExample methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 16:29'!
turnLeft
	self onFwd: 0 speed: 100;
		onFwd: 2 speed: 25! !

!FollowLineExample methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 16:29'!
turnRight
	self onFwd: 2 speed: 100;
		onFwd: 0 speed: 25! !

NxtProgram subclass: #MathTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!MathTest class methodsFor: 'definition' stamp: 'Richo 2/1/2011 13:58'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!MathTest methodsFor: 'as yet unclassified' stamp: 'Richo 2/1/2011 17:19'!
main
	<task>
	(6 isDivisibleBy: 2)
		ifTrue: [self displayText: 'True' x: 0 y: 0].
	6 \\ 2 = 0 ifTrue: [self wait: 500]! !

NxtProgram subclass: #NxcTutorialExample1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxcTutorialExample1 class methodsFor: 'definition' stamp: 'Richo 1/3/2011 15:45'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxcTutorialExample1 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 15:46'!
main
	self onFwd: 0 speed: 75;
		 onFwd: 2 speed: 75.
	self wait: 4000.
	self onRev: 0 speed: 75;
		 onRev: 2 speed: 75.
	self wait: 4000.
	self off: 0; off: 2! !

NxtProgram subclass: #NxcTutorialExample2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxcTutorialExample2 class methodsFor: 'definition' stamp: 'Richo 1/3/2011 16:19'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxcTutorialExample2 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 16:42'!
main
	| moveTime turnTime |
	<tempTypes: 'int int'>
	moveTime := 1000.
	turnTime := 500.
	10
		timesRepeat: [4
				timesRepeat: [self onFwd: 0 speed: 75;
						 onFwd: 2 speed: 75.
					self wait: moveTime.
					self onRev: 2 speed: 75.
					self wait: turnTime]].
	self off: 0; 
		 off: 2! !

NxtProgram subclass: #NxcTutorialExample3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxcTutorialExample3 class methodsFor: 'definition' stamp: 'Richo 1/3/2011 16:44'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxcTutorialExample3 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 16:45'!
main
	
	| moveTime turnTime |
	turnTime := 360.
	moveTime := 200.
	50 timesRepeat: [
		self onFwd: 0 speed: 75; onFwd: 2 speed: 75.
		self wait: moveTime.
		self onRev: 2 speed: 75.
		self wait: turnTime.
		moveTime := moveTime + 200.
		].
	self off: 0; off: 2! !

!NxtProgram class methodsFor: 'translating' stamp: 'Richo 1/2/2011 00:02'!
asNxcProgram
	"Useful method to easily translate a program to nxc. By default the class name is used as program name"
	^ self asNxcProgram: self name! !

!NxtProgram class methodsFor: 'translating' stamp: 'Richo 2/8/2011 16:26'!
asNxcProgram: programName 
	"Useful method to easily translate a program to nxc"
	^ NxcTranslatorViejo new translate: self asProgramNamed: programName! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:38'!
definition
	"Answer a String that defines the receiver."
	"Overwritten to show the #instanceVariablesTypes: message in the class definition of NxtProgram subclasses"
	| aStream path |
	self = NxtProgram ifTrue: [^super definition].
	aStream := WriteStream
				on: (String new: 300).
	superclass == nil
		ifTrue: [aStream nextPutAll: 'ProtoObject']
		ifFalse: [path := ''.
			self environment
				scopeFor: superclass name
				from: nil
				envtAndPathIfFound: [:envt :remotePath | path := remotePath].
			aStream nextPutAll: path , superclass name].
	aStream nextPutAll: self kindOfSubclass;
		 store: self name.
	[(self hasTraitComposition
			and: [self traitComposition notEmpty])
		ifTrue: [aStream cr; tab; nextPutAll: 'uses: '; nextPutAll: self traitCompositionString]] on: MessageNotUnderstood do: ["Ignore DNU. Traits don't exist in Etoys"].
	aStream cr; tab; nextPutAll: 'instanceVariableNames: '; store: self instanceVariablesString.
	aStream cr; tab; nextPutAll: 'instanceVariableTypes: '; store: self instanceVariableTypesString.
	aStream cr; tab; nextPutAll: 'classVariableNames: '; store: self classVariablesString.
	aStream cr; tab; nextPutAll: 'poolDictionaries: '; store: self sharedPoolsString.
	aStream cr; tab; nextPutAll: 'category: '; store: (SystemOrganization categoryOfElement: self name) asString.
	superclass
		ifNil: [aStream nextPutAll: '.';
				 cr.
			aStream nextPutAll: self name.
			aStream space; nextPutAll: 'superclass: nil'].
	^ aStream contents! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:38'!
definitionST80
	"Only useful for the Etoys image"
	self = NxtProgram ifTrue: [^super definitionST80].
	^ self definition! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:51'!
instVarTypes 
	"Returns a collection of symbols representing the nxc type of each inst var in the order ther are defined"
	| instVarTypes |
	instVarTypes := (self instanceVariableTypesString subStrings: {Character space}) collect: [:each | each asSymbol].
	instVarTypes isEmpty ifTrue: [instVarTypes := #()].
	^ instVarTypes ! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:29'!
instanceVariableTypes: aString 
"Automatically compiles a method called #instanceVariableTypesString that returns the string given as a parameter"
self class compile: 'instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you''re doing"
^ ', aString printString, ' withBlanksCondensed' classified: #definition
! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:24'!
instanceVariableTypesString
	"This method will be automaticaly generated for subclasses whenever a subclass of NxtProgram is compiled. It returns the information of the inst vars' type. I choose to do it this way (instead of using an class inst var) to ensure the type info will be saved when we file out this class or store it using Monticello"
	^ ''! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 2/18/2011 13:44'!
subclass: nameOfClass instanceVariableNames: instVarNames instanceVariableTypes: instVarTypes classVariableNames: classVarNames poolDictionaries: poolDictnames category: category1
"Overwritten to allow defining the inst vars' type info of a class"
	^(ClassBuilder new
		superclass: self
		subclass: nameOfClass
		instanceVariableNames: instVarNames
		classVariableNames: classVarNames
		poolDictionaries: poolDictnames
		category: category) instanceVariableTypes: instVarTypes! !

!NxtProgram methodsFor: 'display module' stamp: 'Richo 1/13/2011 18:23'!
clearScreen
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'display module' stamp: 'Richo 1/2/2011 18:01'!
displayNumber: value x: x y: y
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

char NumOut  ( int  x,  
  int  y,  
  variant  value,  
  unsigned long  options = DRAW_OPT_NORMAL   
 )   [inline] 

Draw a number. 

Draw a numeric value on the screen at the specified x and y location. The y value must be a multiple of 8. Valid line number constants are listed in the Line number constants group. Optionally specify drawing options. If this argument is not specified it defaults to DRAW_OPT_NORMAL. Valid display option constants are listed in the Drawing option constants group. 

See also:
SysDrawText, DrawTextType
Parameters:
 x  The x value for the start of the number output.  
 y  The text line number for the number output.  
 value  The value to output to the LCD screen. Any numeric type is supported.  
 options  The optional drawing options.  

Returns:
The result of the drawing operation. 

-------------------------------------------------------------------------------------
Note from Richo: The options parameter is not supported, use #displayNumber:x:y:options: for that."
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'display module' stamp: 'Richo 1/2/2011 18:01'!
displayNumber: value x: x y: y options: options
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

char NumOut  ( int  x,  
  int  y,  
  variant  value,  
  unsigned long  options = DRAW_OPT_NORMAL   
 )   [inline] 

Draw a number. 

Draw a numeric value on the screen at the specified x and y location. The y value must be a multiple of 8. Valid line number constants are listed in the Line number constants group. Optionally specify drawing options. If this argument is not specified it defaults to DRAW_OPT_NORMAL. Valid display option constants are listed in the Drawing option constants group. 

See also:
SysDrawText, DrawTextType
Parameters:
 x  The x value for the start of the number output.  
 y  The text line number for the number output.  
 value  The value to output to the LCD screen. Any numeric type is supported.  
 options  The optional drawing options.  

Returns:
The result of the drawing operation. 

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'display module' stamp: 'Richo 1/4/2011 10:21'!
displayText: value x: x y: y
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/4/2011 10:19'!
float: outputPort
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'math operations' stamp: 'Richo 2/1/2011 17:12'!
floor: n
<returnType: 'long'> <argTypes: 'long'>
<receiver: '@arg1' selector: #floor arguments: #()>
"long floor(long n)
{
    //if we pass a float as parameter it is automatically converted :)
    return n;
}"
^ n! !

!NxtProgram methodsFor: 'math operations' stamp: 'Richo 2/1/2011 17:11'!
foo: t1 and: t2 and: t3
<receiver: '@arg1' selector: #bar:and: arguments: #('@arg2' '@arg3')>
self displayText: 'Bar' x: 0 y: 0; wait: 5000! !

!NxtProgram methodsFor: 'math operations' stamp: 'Richo 2/1/2011 17:13'!
isDivisible: r by: a
<returnType: 'bool'> <argTypes: 'long long'>
<receiver: '@arg1' selector: #isDivisibleBy: arguments: #('@arg2')>
"bool isDivisibleBy(long r, long a)
{
     if(a == 0) {return false;}
     if(mod(r, a) == 0)
     {
         return true;
     }
     else
     {
         return false;
     }
}"
a = 0 ifTrue: [^false].
r \\ a = 0
	ifTrue: [^ true]
	ifFalse: [^ false]! !

!NxtProgram methodsFor: 'math operations' stamp: 'Richo 2/1/2011 17:13'!
mod: r with: a
<returnType: 'long'> <argTypes: 'long long'>
<receiver: '@arg1' selector: #\\ arguments: #('arg2')>
"long mod(long r, long a)
{
    return r - (r / a * a);
}"
^ r - (r / a * a)! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/4/2011 13:24'!
motorPower: outputPort
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/4/2011 13:24'!
motorSpeed: outputPort
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'others' stamp: 'Richo 1/5/2011 15:22'!
nxc: aString
	"This method lets you write nxc code that will be appended to the program without modification"
	self error: 'Just a comment'! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/3/2011 15:46'!
off: outputPort
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/2/2011 18:21'!
onFwd: outputs speed: speed
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

void OnFwd  ( byte  outputs,  
  char  pwr   
 )   [inline] 

Run motors forward. 

Set outputs to forward direction and turn them on.

Parameters:
 outputs  Desired output ports. Can be a constant or a variable, see Output port constants. If you use a variable and want to control multiple outputs in a single call you need to use a byte array rather than a byte and store the output port values in the byte array before passing it into this function.  
 pwr  Output power, 0 to 100. Can be negative to reverse direction.  

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/2/2011 18:21'!
onRev: output speed: speed
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

void OnRev  ( byte  outputs,  
  char  pwr   
 )   [inline] 

Run motors backward. 

Set outputs to reverse direction and turn them on.

Parameters:
 outputs  Desired output ports. Can be a constant or a variable, see Output port constants. If you use a variable and want to control multiple outputs in a single call you need to use a byte array rather than a byte and store the output port values in the byte array before passing it into this function.  
 pwr  Output power, 0 to 100. Can be negative to reverse direction.  

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'sound module' stamp: 'Richo 1/4/2011 10:22'!
playFile: fileName
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'sound module' stamp: 'Richo 1/4/2011 10:22'!
playTone: freq duration: milliseconds
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'sound module' stamp: 'Richo 1/13/2011 14:26'!
playTone: freq duration: milliseconds volume: volume repeat: aBoolean
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'display module' stamp: 'Richo 1/13/2011 18:23'!
resetScreen
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/4/2011 10:20'!
rotateMotor: outputPort speed: speed angle: angle
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'math operations' stamp: 'Richo 2/1/2011 17:12'!
rounded: n
<returnType: 'long'> <argTypes: 'float'>
<receiver: '@arg1' selector: #rounded arguments: #()>
"long rounded(float n)
{
    if(n >= 0)
    {
       return truncated(n + 0.5);
    }
    else
    {
       return truncated(n - 0.5);
    }
}"
n >= 0 
	ifTrue: [^ self truncated: n + 0.5]
	ifFalse: [^ self truncated: n - 0.5]! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
sensorBoolean: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
sensorNormalized: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
sensorRaw: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
sensorScaled: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 13:20'!
sensorType: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

byte SensorType	(	const byte & 	port	 ) 	 [inline]
Read sensor type.

Return the type of a sensor on the specified port.

Parameters:
port 	The sensor port. See Input port constants. A variable whose value is the desired sensor port may also be used.
Returns:
The sensor's type. See Sensor type constants.

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:23'!
sensorUS: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/2/2011 22:06'!
sensorValue: port
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

unsigned int SensorValue  ( const byte &  port  )  [inline] 

Read sensor scaled value. 

Return the processed sensor reading for a sensor on the specified port. This is the same value that is returned by the sensor value names (e.g. SENSOR_1) or the Sensor function.

Parameters:
 port  The sensor port. See Input port constants. A variable whose value is the desired sensor port may also be used.  

Returns:
The sensor's scaled value. 
-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/2/2011 22:06'!
setSensorLight: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

void SetSensorLight  ( const byte &  port,  
  bool  bActive = true   
 )   [inline] 

Configure a light sensor. 

Configure the sensor on the specified port as an NXT light sensor. 

Parameters:
 port  The port to configure. See Input port constants.  
 bActive  A boolean flag indicating whether to configure the port as an active or inactive light sensor. The default value for this optional parameter is true.  
 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/2/2011 22:06'!
setSensorLight: inputPort active: aBoolean
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 void SetSensorLight  ( const byte &  port,  
  bool  bActive = true   
 )   [inline] 

Configure a light sensor. 

Configure the sensor on the specified port as an NXT light sensor. 

Parameters:
 port  The port to configure. See Input port constants.  
 bActive  A boolean flag indicating whether to configure the port as an active or inactive light sensor. The default value for this optional parameter is true.  
-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
setSensorLowSpeed: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
setSensorLowSpeed: inputPort scaling: aBoolean
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:18'!
setSensorSound: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:18'!
setSensorSound: inputPort scaling: aBoolean
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:18'!
setSensorTouch: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'math operations' stamp: 'Richo 2/1/2011 17:12'!
sin: x
<returnType: 'float'> <argTypes: 'float'>
<tempTypes: 'float float float float'>
<receiver: '@arg1' selector: #sin arguments: #()>
"float sin(float x)
{
  float sum = x;
  float delta = x;
  float x2 = 0 - (x * x);
  float i = 2;
  while(delta > 0.000000000001)
  {
    //once
    delta = (delta * x2) / (i * (i + 1));
    i = i + 2;
    sum = sum + delta;
    
    //twice
    delta = (delta * x2) / (i * (i + 1));
    i = i + 2;
    sum = sum + delta;
  }
  return sum;
  
}"
| sum delta x2 i |
sum := x.
delta := x.
x2 := 0 - (x * x).
i := 2.
[delta > 0.000000000001]	 whileTrue: [
	"once"
	delta := (delta * x2) / (i * (i + 1)).
	i := i + 2.
	sum := sum + delta.
	"twice"
	delta := (delta * x2) / (i * (i + 1)).
	i := i + 2.
	sum := sum + delta.].
^ sum! !

!NxtProgram methodsFor: 'math operations' stamp: 'Richo 2/1/2011 17:12'!
truncated: n
<returnType: 'long'> <argTypes: 'long'>
<receiver: '@arg1' selector: #truncated arguments: #()>
"long truncated(long n)
{
    //if we pass a float as parameter it is automatically converted :)
    return n;
}"
^ n! !

!NxtProgram methodsFor: 'command module' stamp: 'Richo 1/2/2011 18:22'!
wait: ms
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

void Wait  ( unsigned long  ms  )  [inline] 

Wait some milliseconds. 

Make a task sleep for specified amount of time (in 1000ths of a second).

Parameters:
 ms  The number of milliseconds to sleep.  

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

NxtProgram subclass: #NxtProgramTest1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest1 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest1 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:22'!
anotherMethod
	self displayNumber: 100 x: 10 y: 50 - (10*5) ! !

!NxtProgramTest1 methodsFor: 'as yet unclassified' stamp: 'Richo 12/29/2010 14:23'!
main
	self onFwd: 0 speed: 100;
		wait: 1000.! !

NxtProgram subclass: #NxtProgramTest10
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest10 class methodsFor: 'definition' stamp: 'Richo 1/5/2011 15:22'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest10 methodsFor: 'as yet unclassified' stamp: 'Richo 2/1/2011 17:31'!
main
	<task>
	self nxc: 'OnFwd(OUT_ABC, 100);';
		 wait: 5000 abs! !

NxtProgram subclass: #NxtProgramTest11
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest11 class methodsFor: 'definition' stamp: 'Richo 1/27/2011 18:42'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest11 methodsFor: 'as yet unclassified' stamp: 'Richo 1/27/2011 18:43'!
main
	self displayText: 'Richo', ' ', 'capo' x: 0 y: 0;
		 wait: 1000! !

NxtProgram subclass: #NxtProgramTest12
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest12 class methodsFor: 'definition' stamp: 'Richo 2/1/2011 15:35'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest12 methodsFor: 'as yet unclassified' stamp: 'Richo 2/1/2011 17:07'!
main
	((6 isDivisibleBy: 2) and: [(5 \\ 3) > 0])
		ifTrue: [self displayNumber: 100 x:0  y: 0].
	5 bar: 6 and: 7
	! !

NxtProgram subclass: #NxtProgramTest2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest2 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest2 methodsFor: 'as yet unclassified' stamp: 'Richo 12/29/2010 14:19'!
main
	| var |
	var := 10 + 20 + (30 * 40).
	self displayNumber: var x: 10 y: 8;
		wait: var;
		onFwd: 0 speed: 100! !

NxtProgram subclass: #NxtProgramTest3
	instanceVariableNames: 'instVar1 instVar2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest3 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ 'byte   char' withBlanksCondensed! !

!NxtProgramTest3 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:59'!
func: arg1 and: arg2
	<argTypes: 'long long'>
	<tempTypes: 'long long'>
	<returnType: 'byte'>
	| temp1 instVar2 |
	instVar2 := instVar1 .
	temp1 := instVar1.
	instVar1 := temp1 * instVar2.
	^ arg1 + arg2 
! !

!NxtProgramTest3 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:59'!
main
	<tempTypes: 'long'>
	| tempVar1 |
	tempVar1 := self func: 3 and: instVar1.
	instVar2 := 4.
	self displayNumber: tempVar1 * instVar2 x: 0 y: 0! !

NxtProgram subclass: #NxtProgramTest4
	instanceVariableNames: 'instVar1 instVar2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest4 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ 'long long' withBlanksCondensed! !

!NxtProgramTest4 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:55'!
func: arg1 and: arg2
	<argTypes: 'long'>
	<returnType: 'int'>
	| temp1 instVar2 |
	instVar2 := instVar1 .
	temp1 := instVar1.
	instVar1 := temp1 * instVar2.
	^ arg1 + arg2 
! !

!NxtProgramTest4 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:56'!
main
	<tempTypes: 'long'>
	| tempVar1 |
	tempVar1 := self func: 3 and: instVar1.
	instVar2 := 4.
	self displayNumber: tempVar1 * instVar2 x: 0 y: 0! !

NxtProgram subclass: #NxtProgramTest5
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest5 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest5 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 14:07'!
main
	
	5 = true
		ifTrue: [self wait: 1000.
			self displayNumber: 500 x: 0 y: 0]
		ifFalse: [self wait: 1200; displayNumber: 500 x: 0 y: 0].
		
	false ifTrue: [self onFwd: 0 speed: 100].
	true ifFalse: [self onFwd: 0 speed: -100]! !

NxtProgram subclass: #NxtProgramTest6
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest6 class methodsFor: 'definition' stamp: 'Richo 1/2/2011 23:56'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest6 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 00:20'!
main
	<tempTypes: 'int'>
	| i |
	i := 1.
	20 timesRepeat: [
	self displayNumber: i x:100  y: 100;
		wait: 1000.
	i := i + 1.
	]! !

NxtProgram subclass: #NxtProgramTest7
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest7 class methodsFor: 'definition' stamp: 'Richo 1/3/2011 16:34'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest7 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 16:39'!
main
	| var |
	var := 1000.
	10 timesRepeat: [
	10 timesRepeat: [
		4 timesRepeat: [
			self onFwd: 0 speed: 100;
				 wait: var.
			]
		].
	]! !

NxtProgram subclass: #NxtProgramTest8
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest8 class methodsFor: 'definition' stamp: 'Richo 1/4/2011 13:24'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest8 methodsFor: 'as yet unclassified' stamp: 'Richo 1/4/2011 13:29'!
main
	[self onFwd: 0 speed: 100.
	self displayNumber: (self motorSpeed: 0) x: 0 y: 0.
	self displayNumber: (self motorPower: 0) x: 0 y: 8.
	self wait: 500] repeat! !

NxtProgram subclass: #NxtProgramTest9
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-ProgramsViejo'!

!NxtProgramTest9 class methodsFor: 'definition' stamp: 'Richo 1/5/2011 12:38'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest9 methodsFor: 'as yet unclassified' stamp: 'Richo 1/5/2011 12:39'!
f
	^ self! !

!NxtProgramTest9 methodsFor: 'as yet unclassified' stamp: 'Richo 1/5/2011 12:38'!
main
	self f! !

Object subclass: #VariableInfo
	instanceVariableNames: 'variables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

!VariableInfo commentStamp: 'Richo 1/1/2011 20:53' prior: 0!
A VariableInfo stores the info (scope, name and type) of all the variables found in a program.

Instance Variables
	variables:		<Dictionary>

variables
	- This dictionary uses the method selectors as keys (and a special key called #'$instVars$' to store variables with a global scope). For each key it stores a set of associations, each association's key being the variable's name and value being its type.!

!VariableInfo methodsFor: 'accessing' stamp: 'Richo 12/31/2010 23:51'!
at: selector
	(variables includesKey: selector) ifFalse: [variables at: selector put: Set new].
	^ variables at: selector ! !

!VariableInfo methodsFor: 'initialize-release' stamp: 'Richo 12/31/2010 22:21'!
initialize
	super initialize.
	variables := Dictionary new! !

!VariableInfo methodsFor: 'accessing' stamp: 'Richo 1/1/2011 21:47'!
instVars
	"I use this weird symbol to avoid a name clash with a method selector"
	^ self at: #'$instVars$'! !

!VariableInfo methodsFor: 'accessing' stamp: 'Richo 1/27/2011 09:43'!
isAnInstVar: varName
	"Looks for a variable with the same name in the instVars set"
	^ self instVars anySatisfy: [:each | each name = varName]! !

!VariableInfo methodsFor: 'accessing' stamp: 'Richo 1/31/2011 16:06'!
typeOf: var in: method
	^ ((self at: method) detect: [:each | each key = var]) value! !

Exception subclass: #InstVarToImplement
	instanceVariableNames: 'ownerClass variable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Exceptions'!

!InstVarToImplement class methodsFor: 'instance creation' stamp: 'Richo 2/21/2011 15:50'!
class: aClass var: aVariableNode 
	^ (self basicNew setClass: aClass var: aVariableNode) initialize.! !

!InstVarToImplement methodsFor: 'exceptionDescription' stamp: 'Richo 2/18/2011 14:01'!
defaultAction
	"No one has handled this error, but now give them a chance to decide how to debug it.  If none handle this either then open debugger (see UnhandedError-defaultAction)"
	UnhandledError signalForException: self! !

!InstVarToImplement methodsFor: 'accessing' stamp: 'Richo 2/18/2011 14:05'!
ownerClass
	^ ownerClass! !

!InstVarToImplement methodsFor: 'private' stamp: 'Richo 2/21/2011 15:51'!
setClass: aClass var: aVariableNode
	ownerClass := aClass.
	variable := aVariableNode! !

!InstVarToImplement methodsFor: 'accessing' stamp: 'Richo 2/21/2011 15:52'!
varName
	^ variable name! !

!InstVarToImplement methodsFor: 'accessing' stamp: 'Richo 2/21/2011 15:52'!
varType
	^ NxcTypeCollector findTypeOf: variable in: ownerClass.! !

Exception subclass: #MethodToImplement
	instanceVariableNames: 'methodClass methodSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Exceptions'!

!MethodToImplement class methodsFor: 'instance creation' stamp: 'Richo 2/21/2011 14:42'!
class: aClass selector: aByteSymbol 
	^ (self basicNew setClass: aClass selector: aByteSymbol) initialize! !

!MethodToImplement methodsFor: 'exceptionDescription' stamp: 'Richo 2/9/2011 11:08'!
defaultAction
	"No one has handled this error, but now give them a chance to decide how to debug it.  If none handle this either then open debugger (see UnhandedError-defaultAction)"
	UnhandledError signalForException: self! !

!MethodToImplement methodsFor: 'printing' stamp: 'Richo 2/21/2011 14:43'!
description
	"Return a textual description of the exception."
	^ 'To implement: ' , methodClass asString , '>>' , methodSelector printString! !

!MethodToImplement methodsFor: 'accessing' stamp: 'Richo 2/21/2011 14:43'!
methodClass
	^ methodClass! !

!MethodToImplement methodsFor: 'accessing' stamp: 'Richo 2/21/2011 14:43'!
methodSelector
	^ methodSelector ! !

!MethodToImplement methodsFor: 'private' stamp: 'Richo 2/21/2011 14:43'!
setClass: aClass selector: aByteSymbol 
	methodClass := aClass.
	methodSelector := aByteSymbol! !
NxcPrimitiveRuleViejo initialize!
